import meteva
import numpy as np
from scipy.spatial import cKDTree
from scipy.interpolate import LinearNDInterpolator
import copy
import pandas as pd

def reset_global_griddata(grd):
    '''
    考虑全球网格展成平面时，有一条经度带不能完全覆盖，为此扩展一列，以覆盖全球
    :param grd:
    :return:
    '''
    grid = meteva.base.get_grid_of_data(grd)
    if  grid.glon[1] + grid.glon[2] - grid.glon[0] -360 > -0.001:
        glon1 = [grid.glon[0],grid.glon[1]+grid.glon[2],grid.glon[2]]
        grid1 = meteva.base.grid(glon1,grid.glat,grid.gtime,grid.dtimes,grid.levels,grid.members)
        grd1 = meteva.base.grid_data(grid1)
        grd1.values[:,:,:,:,:,:-1] = grd.values[:,:,:,:,:,:]
        grd1.values[:, :, :, :, :, -1] = grd.values[:, :, :, :, :, 0]
        return  grd1
    else:
        return grd

def reset_lon_range(grd,sta):
    '''
    经度的取值范围有-180 —— 180 和 0-360 两种方式， 此函数将grd 的取值范围向station 统一
    :param grd:
    :param station:
    :return:
    '''
    grid = meteva.base.get_grid_of_data(grd)
    slon0 = np.min(sta.loc[:, "lon"].values)
    if slon0 < 0 and grid.glon[1] > 180:
        lons0 = grd["lon"].values
        nlon = lons0.size
        index = np.argmax(lons0>180)
        lons1 = np.zeros(nlon)
        lons1[0:nlon - index] = lons0[index:nlon] - 360
        lons1[nlon - index:] = lons0[0:index]
        glon1 = [lons1[0],lons1[-1],lons1[1] - lons1[0]]
        grid1 = meteva.base.grid(glon1,grid.glat,grid.gtime,grid.dtimes,grid.levels,grid.members)
        grd1 = meteva.base.grid_data(grid1)
        grd1.values[:,:,:,:,:,0:nlon - index] = grd.values[:,:,:,:,:,index:nlon]
        grd1.values[:, :, :, :, :, nlon - index:] = grd.values[:, :, :, :, :, 0:index]
        return grd1
    else:
        elon0 = np.max(sta.loc[:, "lon"].values)
        if elon0 >180 and grid.glon[0] <0:
            lons0 = grd["lon"].values
            nlon = lons0.size
            index = np.argmax(lons0 >=0)
            lons1 = np.zeros(nlon)
            lons1[0:nlon - index] = lons0[index:nlon]
            lons1[nlon - index:] = lons0[0:index] + 360
            grid1 = meteva.base.grid(lons1, grid.glat, grid.gtime, grid.dtimes, grid.levels, grid.members)
            grd1 = meteva.base.grid_data(grid1)
            grd1.values[:, :, :, :, :, 0:nlon - index] = grd.values[:, :, :, :, :, index:nlon]
            grd1.values[:, :, :, :, :, nlon - index:] = grd.values[:, :, :, :, :, 0:index]
            return grd1
        else:
            return grd


#格点到站点的插值
def interp_gs_nearest(grd,sta,used_coords = "xy"):
    '''
    :param grd:
    :param sta:
    :return:
    '''
    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    column_list1 = ['lon', 'lat']
    column_list1.extend(members)
    column_list2 = ['level','time','dtime','id', 'lon', 'lat']
    column_list2.extend(members)
    grid = meteva.base.get_grid_of_data(grd)

    if used_coords == "xy":
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        if len(sta1.index)==0:return None
        ig = np.round((sta1.loc[:,'lon'].values - grid.slon) / grid.dlon).astype(dtype = 'int16')
        jg = np.round((sta1.loc[:,'lat'].values - grid.slat) / grid.dlat).astype(dtype = 'int16')
        sta_list = []
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    sta = sta1.loc[:,["time","id","lon","lat"]]
                    sta.loc[:,'time'] = times[j]
                    sta.loc[:,'dtime'] = dtimes[k]
                    sta.loc[:,'level'] = levels[i]
                    for m in range(len(members)):
                        dat = grd.values[m,i,j,k,:,:]
                        dat_sta= dat[jg,ig]
                        sta.loc[:,members[m]] = dat_sta
                    sta_list.append(sta)
        #sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = pd.concat(sta_list,axis = 0)
        sta_all = sta_all.reindex(columns=column_list2)
        return sta_all


#格点到站点的插值,线性
def interp_gs_linear(grd,sta,used_coords = "xy"):

    grd = reset_lon_range(grd,sta)
    grd = reset_global_griddata(grd)

    #print("**0")
    levels = copy.deepcopy(grd["level"].values)
    times = copy.deepcopy(grd["time"].values)
    dtimes = copy.deepcopy(grd["dtime"].values)
    members = copy.deepcopy(grd["member"].values)
    column_list1 = ['lon', 'lat']
    column_list1.extend(members)
    column_list2 = ['level','time','dtime','id', 'lon', 'lat']
    column_list2.extend(members)
    grid = meteva.base.get_grid_of_data(grd)
    if used_coords == "xy":
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        ig = ((sta1['lon'].values - grid.slon) // grid.dlon).astype(dtype = 'int16')
        jg = ((sta1['lat'].values - grid.slat) // grid.dlat).astype(dtype = 'int16')
        dx = (sta1['lon'].values - grid.slon) / grid.dlon - ig
        dy = (sta1['lat'].values - grid.slat) / grid.dlat - jg
        c00 = (1 - dx) * (1 - dy)
        c01 = dx * (1 - dy)
        c10 = (1-dx) * dy
        c11 = dx * dy
        ig1 = np.minimum(ig + 1, grid.nlon - 1)
        jg1 = np.minimum(jg + 1, grid.nlat - 1)
        sta_list = []
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    sta = sta1.loc[:,[ "id", "lon", "lat"]]
                    sta['time'] = times[j]
                    sta.loc[:,'dtime'] = dtimes[k]
                    sta.loc[:,'level'] = levels[i]
                    for m in range(len(members)):
                        dat = grd.values[m,i,j,k,:,:]
                        dat_sta= c00 * dat[jg,ig] + c01 * dat[jg,ig1] + c10 * dat[jg1,ig] + c11 * dat[jg1,ig1]
                        sta.loc[:,members[m]] = dat_sta
                    sta_list.append(sta)
        #sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = pd.concat(sta_list, axis=0)
        sta_all = sta_all.reindex(columns=column_list2)
        return sta_all

    elif used_coords == "xyz":
        sta1 = meteva.base.sele.in_grid_xyz(sta, grid)
        lev_s = sta1["level"].values[:]
        lev_g = np.array(grid.levels)
        kg = meteva.base.tool.math_tools.get_index(lev_s,grid.levels)
        kg1 = np.minimum(kg + 1, len(grid.levels) - 1)
        ig = ((sta1['lon'] - grid.slon) // grid.dlon).astype(dtype='int16')
        jg = ((sta1['lat'] - grid.slat) // grid.dlat).astype(dtype='int16')
        dx = (sta1['lon'] - grid.slon) / grid.dlon - ig
        dy = (sta1['lat'] - grid.slat) / grid.dlat - jg
        dz = (lev_s - lev_g[kg])/(lev_g[kg1] - lev_g[kg] + 1e-30)
        c00 = (1 - dx) * (1 - dy)
        c01 = dx * (1 - dy)
        c10 = (1 - dx) * dy
        c11 = dx * dy
        ig1 = np.minimum(ig + 1, grid.nlon - 1)
        jg1 = np.minimum(jg + 1, grid.nlat - 1)
        sta_all = None
        for j in range(len(times)):
            for k in range(len(dtimes)):
                sta = sta1.loc[:,["level","id","lon","lat"]]
                sta.loc[:,'time'] = times[j]
                sta.loc[:,'dtime'] = dtimes[k]
                for m in range(len(members)):
                    dat = grd.values[m,:,j,k,:,:]
                    dat_sta= (c00 * dat[kg,jg,ig] + c01 * dat[kg,jg,ig1] + c10 * dat[kg,jg1,ig] + c11 * dat[kg,jg1,ig1]) * (1-dz)
                    dat_sta += (c00 * dat[kg1, jg, ig] + c01 * dat[kg1, jg, ig1] + c10 * dat[kg1, jg1, ig] + c11 * dat[
                        kg1, jg1, ig1]) *  dz
                    sta.loc[:,members[m]] = dat_sta
                sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = sta_all.reindex(columns=column_list2)
        return sta_all

    elif used_coords =="xydt":
        if(len(grid.dtimes) == 1):
            print("dtime维度size = 1,无法开展dtime维度插值")
            return
        '''
        还有问题
        '''
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        sta1 = meteva.base.sele.between_dtime_range(sta1,grid.dtimes[0],grid.dtimes[-1])
        dtime_g = np.array(grid.dtimes)
        lon_g = grd["lon"].values
        lat_g = grd["lat"].values

        coords_g = np.zeros((dtime_g.size, lat_g.size, lat_g.size, 3))
        coords_g[..., 0] = dtime_g.reshape((dtime_g.size,1, 1))
        coords_g[..., 1] = lat_g.reshape((1, lat_g.size, 1))
        coords_g[..., 2] = lon_g.reshape((1,1, lon_g.size))
        coords_g = coords_g.reshape((dtime_g.size * lat_g.size * lat_g.size, 3))

        coords_s = np.zeros((len(sta1.index), 3))
        coords_s[:, 0] = sta1["dtime"].values
        coords_s[:, 1] = sta1["lat"].values
        coords_s[:, 2] = sta1["lon"].values
        sta_all = None
        for j in range(len(times)):
            for k in range(len(levels)):
                sta = sta1.loc[:,["dtime","id","lon","lat"]]
                sta.loc[:,'time'] = times[j]
                sta.loc[:,'level'] = levels[k]
                for m in range(len(members)):
                    dat_g = grd.values[m,k,j,:,:,:]
                    interpolator = LinearNDInterpolator(coords_g, dat_g.reshape((dat_g.size)))
                    dat_s = interpolator(coords_s)
                    sta.loc[:, members[m]] = dat_s
                sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = sta_all.reindex(columns=column_list2)
        return sta_all


#格点到站点的插值，三次
def interp_gs_cubic(grd,sta,used_coords = "xy"):
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    column_list1 = ['lon', 'lat']
    column_list1.extend(members)
    column_list2 = ['level','time','dtime','id', 'lon', 'lat']
    column_list2.extend(members)
    grid = meteva.base.get_grid_of_data(grd)
    sta_all = None
    if used_coords == "xy":
        sta1 = meteva.base.sele.in_grid_xy(sta, grid)
        ig = ((sta1['lon'] - grid.slon) // grid.dlon).astype(dtype='int16')
        jg = ((sta1['lat'] - grid.slat) // grid.dlat).astype(dtype='int16')
        dx = (sta1['lon'] - grid.slon) / grid.dlon - ig
        dy = (sta1['lat'] - grid.slat) / grid.dlat - jg
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    sta = sta1.loc[:,["id","lon","lat"]]
                    sta.loc[:,'time'] = times[j]
                    sta.loc[:,'dtime'] = dtimes[k]
                    sta.loc[:,'level'] = levels[i]
                    for m in range(len(members)):
                        dat = grd.values[m,i,j,k,:,:]
                        sum = np.zeros(len(sta.index))
                        for p in range(-1, 3, 1):
                            iip = np.minimum(np.maximum(ig + p, 0), grid.nlon - 1)
                            fdx = cubic_f(p, dx)
                            for q in range(-1, 3, 1):
                                jjq = np.minimum(np.maximum(jg + q, 0), grid.nlat - 1)
                                fdy = cubic_f(q, dy)
                                fdxy = fdx * fdy
                                sum[:] += fdxy * dat[jjq, iip]
                        sta.loc[:, members[m]] =sum
                    sta_all = meteva.base.combine_join(sta_all,sta)
        sta_all = sta_all.reindex(columns=column_list2)
        return sta_all


    grid = meteva.base.basicdata.get_grid_of_data(grd)
    sta1 = meteva.base.sele.in_grid_xy(sta, grid)
    dat0 = grd.values
    dat = np.squeeze(dat0)
    ig = ((sta1['lon'] - grid.slon) // grid.dlon).astype(dtype = 'int16')
    jg = ((sta1['lat'] - grid.slat) // grid.dlat).astype(dtype = 'int16')
    dx = (sta1['lon'] - grid.slon) / grid.dlon - ig
    dy = (sta1['lat'] - grid.slat) / grid.dlat - jg
    data_name = meteva.base.get_stadata_names(sta1)[0]
    for p in range(-1,3,1):
        iip = np.minimum(np.maximum(ig+p,0),grid.nlon-1)
        fdx = cubic_f(p, dx)
        for q in range(-1,3,1):
            jjq = np.minimum(np.maximum(jg+q,0),grid.nlat-1)
            fdy = cubic_f(q,dy)
            fdxy = fdx * fdy
            sta1[data_name] +=  fdxy * dat[jjq,iip]
    sta1['time'] = grid.stime
    sta1['dtime'] = grid.dtimes[0]
    sta1['level'] = grid.levels[0]
    meteva.base.basicdata.set_stadata_names(sta1,grid.members)
    return sta1


def interp_sg_idw(sta0, grid, background=None, effectR=1000, nearNum=8,decrease = 2):

    sta1 = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta_list = meteva.base.split(sta1,["member","level","time","dtime"])
    grd_list = []
    for sta in sta_list:
        data_name = meteva.base.get_stadata_names(sta)
        index0 = sta.index[0]
        dtime = sta.loc[index0, 'dtime'].astype(int)
        level = sta.loc[index0, 'level'].astype(int)
        grid2 = meteva.base.basicdata.grid(grid.glon, grid.glat, [sta.loc[index0, 'time']],
                                                           [dtime],
                                                           [level], data_name)
        xyz_sta = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,
                                                                                    sta['lat'].values,
                                                                                    R=meteva.base.basicdata.const.ER)
        lon = np.arange(grid2.nlon) * grid2.dlon + grid2.slon
        lat = np.arange(grid2.nlat) * grid2.dlat + grid2.slat
        grid_lon, grid_lat = np.meshgrid(lon, lat)
        xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(),
                                                                                     grid_lat.flatten(),
                                                                                     R=meteva.base.basicdata.const.ER)
        tree = cKDTree(xyz_sta)
        # d,inds 分别是站点到格点的距离和id
        if nearNum > len(sta.index):
            nearNum = len(sta.index)
        d, inds = tree.query(xyz_grid, k=nearNum)
        if nearNum >1:
            d += 1e-6
            w = 1.0 / d ** decrease
            input_dat = sta.values[:,-1]
            dat = np.sum(w * input_dat[inds], axis=1) / np.sum(w, axis=1)
            bg = meteva.base.basicdata.grid_data(grid2)
            if (background is not None):
                bg = interp_gg_linear(background, grid2)
            bg_dat = bg.values.flatten()
            dat = np.where(d[:, 0] > effectR, bg_dat, dat)
        else:
            input_dat = sta.iloc[:,-1].values
            dat = input_dat[inds]
            bg = meteva.base.basicdata.grid_data(grid2)
            if (background is not None):
                bg = interp_gg_linear(background, grid2)
            bg_dat = bg.values.flatten()
            dat = np.where(d[:] > effectR, bg_dat, dat)
        dat = dat.astype(np.float32)
        grd = meteva.base.basicdata.grid_data(grid2, dat)
        grd.name = data_name[0]
        grd_list.append(grd)

    grd_all = meteva.base.concat(grd_list)
    return grd_all


def interp_sg_idw_delta(sta0, grid,  halfR=1000, nearNum=8,decrease = 2):
    sta1 = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta_list = meteva.base.split(sta1,["member","level","time","dtime"])
    grd_list = []
    for sta in sta_list:
        data_name = meteva.base.get_stadata_names(sta)
        index0 = sta.index[0]
        grid2 = meteva.base.basicdata.grid(grid.glon, grid.glat, [sta.loc[index0, 'time']],
                                                           [sta.loc[index0, 'dtime']],
                                                           [sta.loc[index0, 'level']], data_name)
        xyz_sta = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,
                                                                                    sta['lat'].values,
                                                                                    R=meteva.base.basicdata.const.ER)
        lon = np.arange(grid2.nlon) * grid2.dlon + grid2.slon
        lat = np.arange(grid2.nlat) * grid2.dlat + grid2.slat
        grid_lon, grid_lat = np.meshgrid(lon, lat)
        xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(),
                                                                                     grid_lat.flatten(),
                                                                                     R=meteva.base.basicdata.const.ER)
        tree = cKDTree(xyz_sta)
        # d,inds 分别是站点到格点的距离和id
        if nearNum > len(sta.index):
            nearNum = len(sta.index)
        d, inds = tree.query(xyz_grid, k=nearNum)
        if nearNum >1:
            d += 1e-6
            w1 = 1.0 / d ** decrease
            w2 = np.exp(-(d/halfR)**2)
            input_dat = sta.values[:,-1]
            dat = np.sum(w1 * w2 * input_dat[inds], axis=1) / np.sum(w1, axis=1)
        else:
            input_dat = sta0.iloc[:,-1].values
            w2 = np.exp(-d/halfR)
            dat = w2 * input_dat[inds]
        dat = dat.astype(np.float32)
        grd = meteva.base.basicdata.grid_data(grid2, dat)
        grd.name = data_name[0]
        grd_list.append(grd)
    grd_all = meteva.base.concat(grd_list)
    return grd_all


def interp_sg_cressman(sta0, grid, r_list,background=None , nearNum=100):
    sta1 = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta_list = meteva.base.split(sta1,["member","level","time","dtime"])
    grd_list = []
    for sta in sta_list:
        data_name = meteva.base.get_stadata_names(sta)
        index0 = sta.index[0]
        grid2 = meteva.base.basicdata.grid(grid.glon, grid.glat, [sta.loc[index0, 'time']],
                                                           [sta.loc[index0, 'dtime']],
                                                           [sta.loc[index0, 'level']], data_name)
        xyz_sta = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,
                                                                                    sta['lat'].values,
                                                                                    R=meteva.base.basicdata.const.ER)
        lon = np.arange(grid2.nlon) * grid2.dlon + grid2.slon
        lat = np.arange(grid2.nlat) * grid2.dlat + grid2.slat
        grid_lon, grid_lat = np.meshgrid(lon, lat)
        xyz_grid = meteva.base.tool.math_tools.lon_lat_to_cartesian(grid_lon.flatten(),
                                                                                     grid_lat.flatten(),
                                                                                     R=meteva.base.basicdata.const.ER)
        tree = cKDTree(xyz_sta)
        # d,inds 分别是站点到格点的距离和id
        nsta = len(sta.index)
        if nearNum > nsta:
            nearNum = nsta
        d, inds = tree.query(xyz_grid, k=nearNum)
        d += 1e-6
        bg = meteva.base.basicdata.grid_data(grid2)
        if (background is not None):
            bg = interp_gg_linear(background, grid2)

        bg_dat = bg.values.flatten()
        input_dat = sta.values[:, -1]

        d2 = d ** 2
        for R in r_list:
            index_in = np.where(d[:,0] < R)[0]
            inds_in = inds[index_in,:]
            r2 = R ** 2
            d2_in = d2[index_in,:]
            w = (r2 -  d2_in)/(r2 + d2_in)
            w[w <0] = 0
            dat = np.sum(w * input_dat[inds_in], axis=1) / np.sum(w, axis=1)
            bg_dat[index_in] = dat[:]
        grd = meteva.base.basicdata.grid_data(grid2, bg_dat)
        grd.name = data_name[0]
        grd_list.append(grd)
    grd_all = meteva.base.concat(grd_list)
    return grd_all


def interp_ss_idw(sta0, station, effectR=1000, nearNum=8,decrease = 2,defalut_value =0):
    '''
    :param sta0: 包含原始数据的站点数据
    :param station: 插值后的目标站点
    :param effectR: 反距离插值最大半径
    :param nearNum: 插值所选用的最近点个数
    :return:
    '''
    sta = meteva.base.sele_by_para(sta0,drop_IV=True)
    sta1 = station.copy()
    xyz_sta0 = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta['lon'].values,sta['lat'].values,R=meteva.base.basicdata.const.ER)
    xyz_sta1 = meteva.base.tool.math_tools.lon_lat_to_cartesian(sta1['lon'].values,sta1['lat'].values,R=meteva.base.basicdata.const.ER)
    tree = cKDTree(xyz_sta0)
    d, inds = tree.query(xyz_sta1, k=nearNum)
    if nearNum >1:
        d += 1e-6
        w = 1.0 / d ** decrease
        input_dat = sta0.iloc[:,-1].values
        dat = np.sum(w * input_dat[inds], axis=1) / np.sum(w, axis=1)
        dat[:] = np.where(d[:,0] > effectR,defalut_value,dat[:])
        sta1.iloc[:,-1] = dat[:]
    else:
        input_dat = sta0.iloc[:,-1].values
        dat = input_dat[inds]
        dat[:] = np.where(d[:] > effectR,defalut_value,dat[:])
        sta1.iloc[:,-1] = dat[:]
    return sta1


def interp_gg_linear(grd, grid,used_coords = "xy",outer_value = None):
    '''
    格点到格点插值
    :param grd:左边的网格数据信息
    :param grid :右边的网格数据信息
    :other_info:网格数据除了xy方向的数值之外，还有time,dtime，leve member 等维度的值，如果other_info= 'left’则返回结果中这些维度的值就采用grd里的值，
    否则采用grid里的值，默认为：left
    :return:双线性插值之后的结果
    '''
    if (grd is None):
        return None
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    icycle = int(360/grid0.dlon)
    iscycle = (grid0.dlon * grid0.nlon >= 360)
    if used_coords == "xy":
        is_out = False
        if not iscycle:
            if (grid.elon > grid0.elon or grid.slon < grid0.slon or grid.elat > grid0.elat or grid.slat < grid0.slat):
                if outer_value is None:
                    print("当目标网格超出数据网格时，outer_value参数必须赋值")
                    return None
                is_out = True

        if is_out:
            grid_new0 = meteva.base.get_inner_grid(grid,grid0)
            grid_new = meteva.base.grid(grid_new0.glon, grid_new0.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        else:
            grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        grd_new = meteva.base.grid_data(grid_new)
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    for m in range(len(members)):
                        # 六维转换为二维的值
                        dat = grd.values[m,i,j,k,:,:]
                        #插值处理
                        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
                        ig = x[:].astype(dtype='int16')
                        dx = x - ig
                        y = (np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat - grid0.slat) / grid0.dlat
                        jg = y[:].astype(dtype='int16')
                        dy = y - jg
                        ii, jj = np.meshgrid(ig, jg)
                        if iscycle:
                            ii1 = ii+1
                            ii = ii%icycle
                            ii1 = ii1%icycle
                        else:
                            ii1 = np.minimum(ii + 1, grid0.nlon - 1)
                        jj1 = np.minimum(jj + 1, grid0.nlat - 1)
                        ddx, ddy = np.meshgrid(dx, dy)
                        c00 = (1 - ddx) * (1 - ddy)
                        c01 = ddx * (1 - ddy)
                        c10 = (1 - ddx) * ddy
                        c11 = ddx * ddy
                        dat2 = (c00 *dat[jj, ii] + c10 * dat[jj1, ii] + c01 * dat[jj, ii1] + c11 * dat[jj1, ii1])
                        grd_new.values[m,i,j,k,:,:] = dat2
        if is_out:
            grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
            grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
    return grd_new



def interp_gg_cubic(grd, grid,used_coords = "xy",outer_value = None):
    '''
    格点到格点插值
    :param grd:左边的网格数据信息
    :param grid :右边的网格数据信息
    :other_info:网格数据除了xy方向的数值之外，还有time,dtime，leve member 等维度的值，如果other_info= 'left’则返回结果中这些维度的值就采用grd里的值，
    否则采用grid里的值，默认为：left
    :return:双线性插值之后的结果
    '''
    if (grd is None):
        return None
    levels = grd["level"].values
    times = grd["time"].values
    dtimes = grd["dtime"].values
    members = grd["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(grd)
    icycle = int(360/grid0.dlon)
    iscycle = (grid0.dlon * grid0.nlon >= 360)
    if used_coords == "xy":
        is_out = False
        if not iscycle:
            if (grid.elon > grid0.elon or grid.slon < grid0.slon or grid.elat > grid0.elat or grid.slat < grid0.slat):
                if outer_value is None:
                    print("当目标网格超出数据网格时，outer_value参数必须赋值")
                    return None
                is_out = True

        if is_out:
            grid_new0 = meteva.base.get_inner_grid(grid,grid0)
            grid_new = meteva.base.grid(grid_new0.glon, grid_new0.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        else:
            grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
        grd_new = meteva.base.grid_data(grid_new)
        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    for m in range(len(members)):
                        # 六维转换为二维的值


                        dat = grd.values[m,i,j,k,:,:]
                        #插值处理
                        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
                        ig = x[:].astype(dtype='int16')
                        dx = x - ig
                        y = (np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat - grid0.slat) / grid0.dlat
                        jg = y[:].astype(dtype='int16')
                        dy = y - jg
                        ii, jj = np.meshgrid(ig, jg)
                        if iscycle:
                            ii1 = ii+1
                            ii = ii%icycle
                            ii1 = ii1%icycle
                        else:
                            ii1 = np.minimum(ii + 1, grid0.nlon - 1)
                        jj1 = np.minimum(jj + 1, grid0.nlat - 1)
                        ddx, ddy = np.meshgrid(dx, dy)

                        for p in range(-1, 3, 1):
                            iip = np.minimum(np.maximum(ii + p, 0), grid0.nlon - 1)
                            fdx = cubic_f(p, dx)
                            for q in range(-1, 3, 1):
                                jjq = np.minimum(np.maximum(jj + q, 0), grid0.nlat - 1)
                                fdy = cubic_f(q, dy)
                                fdxx, fdyy = np.meshgrid(fdx, fdy)
                                fdxy = fdxx * fdyy
                                grd_new.values[m,i,j,k,:,:] += fdxy * dat[jjq, iip]

        if is_out:
            grid_new1 = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
            grd_new = meteva.base.expand_to_contain_another_grid(grd_new,grid_new1,outer_value=outer_value)
    return grd_new


def interp_xg_linear(dataArray,grid,used_coords = "xy"):
    meteva.base.reset(dataArray)
    levels = dataArray["level"].values
    times = dataArray["time"].values
    dtimes = dataArray["dtime"].values
    members = dataArray["member"].values
    grid0 = meteva.base.basicdata.get_grid_of_data(dataArray)
    grid_new = meteva.base.grid(grid.glon, grid.glat, grid0.gtime, grid0.dtimes, grid0.levels, grid0.members)
    grd_new = meteva.base.grid_data(grid_new)


    if used_coords == "xy":

        # 插值处理
        x = ((np.arange(grid_new.nlon) * grid_new.dlon + grid_new.slon - grid0.slon) / grid0.dlon)
        ig = x[:].astype(dtype='int16')
        dx = x - ig
        y = np.arange(grid_new.nlat) * grid_new.dlat + grid_new.slat
        nlat = len(y)
        jg = np.zeros(nlat).astype(dtype='int16')
        dy = np.zeros(nlat)
        lats_old = dataArray.lat.values
        for ny in range(nlat):
            js = 0
            for j in range(js,nlat-1):
                if y[ny] >= lats_old[j] and y[ny] <= lats_old[j+1]:
                    jg[ny] = j
                    dy[ny] = (y[ny] -  lats_old[j])/(lats_old[j+1] - lats_old[j])
                    js = j
                    break


        #print(jg)
        ii, jj = np.meshgrid(ig, jg)
        ii1 = np.minimum(ii + 1, grid0.nlon - 1)
        jj1 = np.minimum(jj + 1, grid0.nlat - 1)
        ddx, ddy = np.meshgrid(dx, dy)
        c00 = (1 - ddx) * (1 - ddy)
        c01 = ddx * (1 - ddy)
        c10 = (1 - ddx) * ddy
        c11 = ddx * ddy


        for i in range(len(levels)):
            for j in range(len(times)):
                for k in range(len(dtimes)):
                    for m in range(len(members)):
                        # 六维转换为二维的值
                        dat = dataArray.values[m, i, j, k, :, :]
                        dat2 = (c00 * dat[jj, ii] + c10 * dat[jj1, ii] + c01 * dat[jj, ii1] + c11 * dat[jj1, ii1])
                        grd_new.values[m, i, j, k, :, :] = dat2

    return grd_new


def cubic_f(n, dx):
    if (n == -1):
        return -dx * (dx - 1) * (dx - 2) / 6
    elif (n == 0):
        return (dx + 1) * (dx - 1) * (dx - 2) / 2
    elif (n == 1):
        return -(dx + 1) * dx * (dx - 2) / 2
    else:
        return (dx + 1) * dx * (dx - 1) / 6



#!/usr/bin/python
# -*- coding: utf-8 -*-
#Filename: loglib.py

'''
Created on Aug 23, 2020

@author: anduin
'''

import logging
import logging.handlers
#from .clienthandler import ClientHandler

class LogLib:
    '''
    日志格式‘%(asctime)s %(levelname)s %(module)s %(lineno)d %(message)s’
    
    使用TimedRotatingFileHandler时，保持240个日志备份，每小时一个日志文件。
    使用ConcurrentRotatingFileHandler时，保持100个1MB的文件。
    上面两个handle仅使用一个。
    
    使用前调用init初始化
    使用LogLib.logger中的debug、info、warn、error、critical写日志
    系统结束后调用unint
    
    需要向每个client发送日志时，要求日志的msg部分，以clientid+空格开始。
    ClientHandle中会取msg第一个空格前的字符串，转换成clientid，所以需要避免以数字字符+空格开始的日志信息，这样会导致信息分发错误。
    '''
    
    logger = None
    #for file log
    fhdlr = None
    #for client log
    chdlr = None
    
    @staticmethod
    def init():
        LogLib.logger = logging.getLogger()
        
        LogLib.logger.setLevel(logging.DEBUG)
        
        return True
    
    @staticmethod
    def addTimedRotatingFileHandler(filename):
        try:
            if __name__ == '__main__':
                #for test
                LogLib.fhdlr = logging.handlers.TimedRotatingFileHandler(filename, 'm', 1, 3)
            else:
                LogLib.fhdlr = logging.handlers.TimedRotatingFileHandler(filename, 'h', 1, 240)
                
            formatter = logging.Formatter('%(asctime)s %(levelname)s %(module)s %(lineno)d %(message)s')
            LogLib.fhdlr.setFormatter(formatter)
            LogLib.logger.addHandler(LogLib.fhdlr)
            return True
        except Exception as data:
            print('LogLib addTimedRotatingFileHandler exception:%s' % (str(data)))
            return False
        

    @staticmethod
    def writeToFile():
        if LogLib.logger == None or LogLib.fhdlr == None:
            return
        try:
            LogLib.fhdlr.flush()
        except Exception as data:
            print('log flush error %s' % (str(data)))
        
    @staticmethod
    def uninit():
        """
        Perform any cleanup actions in the logging system (e.g. flushing buffers).
        Should be called at application exit.
        """
        try:
            if LogLib.fhdlr != None:
                LogLib.fhdlr.flush()
        except Exception as data:
            print('file log flush error %s' % (str(data)))
            
        try:
            if LogLib.fhdlr != None:
                LogLib.fhdlr.close()
        except Exception as data:
            print('file log close error %s' % (str(data)))
                
        try:
            if LogLib.chdlr != None:
                LogLib.chdlr.close()
        except Exception as data:
            print('client log close error %s' % (str(data)))
                
        if LogLib.fhdlr != None:
            LogLib.logger.removeHandler(LogLib.fhdlr)
            LogLib.fhdlr = None
        
        if LogLib.chdlr != None:
            LogLib.logger.removeHandler(LogLib.chdlr)
            LogLib.chdlr = None
        
        LogLib.logger = None
        
    @staticmethod
    def debug(msg):
        if LogLib.logger is None:
            return

        LogLib.logger.debug(msg)
        
    @staticmethod
    def info(msg):
        if LogLib.logger is None:
            return

        LogLib.logger.info(msg)
        
    @staticmethod
    def warning(msg):
        if LogLib.logger is None:
            return

        LogLib.logger.warning(msg)
        
    @staticmethod
    def warn(msg):
        if LogLib.logger is None:
            return

        LogLib.logger.warn(msg)

    @staticmethod
    def error(msg):
        if LogLib.logger is None:
            return

        LogLib.logger.error(msg)
        
    @staticmethod
    def exception(msg, exc_info=True):
        if LogLib.logger is None:
            return

        LogLib.logger.exception(msg, exc_info=exc_info)
        
    @staticmethod
    def critical(msg):
        if LogLib.logger is None:
            return

        LogLib.logger.critical(msg)
        
#for test
if __name__ == '__main__':
    import time
    #from dispatch.clientmgr import ClientMgr
    
    LogLib.init()
    #LogLib.addTimedRotatingFileHandler('./testlog')
    #cm = ClientMgr()
    #LogLib.addClientHandle(cm)
    
    for i in range(0, 400):
        #LogLib.write(logging.INFO, 'test')
        LogLib.logger.debug('test')
        time.sleep(1)
        
    LogLib.uninit()
    
    print('test done')
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Jan 23 11:29:25 2022

@author: tangbuxing
"""
import numpy as np
import pandas as pd
from sklearn import utils
from sklearn.utils import resample
from sklearn import metrics
from scipy import stats

# boot_CI(booted, conf = 1-alpha, Type = "bac", index = i)
def bootstrap_CI(actual_list, predict_list, num_repeats=1000, stat='roc_auc',
                 confident_lvl=0.95, side='two', random_state=0):
    assert len(actual_list) == len(predict_list)
    try:
        all_stats = []
        for i in range(num_repeats):
            actual_list_resampled, predict_list_resampled = resample(actual_list, predict_list)
            if stat == 'roc_auc':
                #计算二分类
                cur_roc_auc = metrics.roc_auc_score(actual_list_resampled, predict_list_resampled)
                all_stats.append(cur_roc_auc)
        roc_auc_left = np.percentile(all_stats, (1 - confident_lvl) / 2. * 100)
        roc_auc_right = np.percentile(all_stats, (1 + confident_lvl) / 2. * 100)

    except Exception as e:
        # print e
        roc_auc_left, roc_auc_right = float('nan'), float('nan')

    return roc_auc_left, roc_auc_right

if __name__ == '__main__':
    aa = [9, 8, 7, 6, 5, 11, 12, 13, 14, 15, 16]
    bb = [1, 3, 5, 7, 9, 2, 4, 6, 8, 9, 22]
    cc = bootstrap_CI(actual_list = aa, predict_list = bb)

    #data = pd.read_csv(r"/Users/tangbuxing/Desktop/tra_test-SAL/field signficance/data/booted001.csv")
    data = Z
    df = len(data) - 1
    ci = stats.t.interval(alpha = 0.95, df = data, loc=np.mean(data), scale=stats.sem(data))






import os
import json
import traceback
import numpy as np
import pandas as pd
import meteva
import matplotlib.pyplot as plt
plt.rcParams['font.sans-serif']=['SimHei'] #用来正常显示中文标签
plt.rcParams['axes.unicode_minus']=False #用来正常显示负号
import xarray as xr

def set_jar_path(path):
    if os.path.exists(path):
        meteva.base.ws_jar_path = path
    else:
        print(path + " not exists")

def java_class_func(jar_path, class_name, func_name, jvm_path=None, *args):
    import jpype
    """
    调用jar包中class下的方法
    :return:
    """
    # jar包路径的配置
    # jarpath = os.path.join(os.path.abspath("."), "D:\\hf-0.1.jar")
    jarpath = os.path.join(os.path.abspath(".."), jar_path)
    # 这里指定了jvm
    if jvm_path:
        jvmpath = jvm_path
    else:
        jvmpath = jpype.getDefaultJVMPath()

    try:
        jpype.startJVM(jvmpath, "-ea", "-Djava.class.path=%s" % jarpath)
    except Exception as e:
        pass

    java_class = jpype.JClass(class_name)
    ru_param = ','.join(list(map(lambda x: json.dumps(x), args)))

    res = str(eval("java_class.%s(%s)" % (func_name, ru_param)))
    #jpype.shutdownJVM()
    return res


def high_low_center(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100,grade_interval = 5):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"high_low_center",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,"grade_interval":grade_interval,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy


def vortex(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"vortex",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy



def trough(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"trough",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy


def reverse_trough(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"reverse_trough",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)
    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy


def convergence_line(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"convergence_line",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy


def shear(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"shear",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy


def jet(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100,jet_min_speed = 12,only_south_jet =False):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"jet",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,"jet_min_speed":jet_min_speed,"only_south_jet":str(only_south_jet),
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy



def subtropical_high(grd,output_dir_root,smooth_times = 0,min_size = 500,necessary_height = 5840,sufficient_height = 5880):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"subtropical_high",
            "smooth_times":smooth_times,"min_size":min_size,"necessary_height":necessary_height,"sufficient_height":sufficient_height,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy


def south_asia_high(grd,output_dir_root,smooth_times = 0,min_size = 800,sn_height = 16800):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    para = {"type":"south_asia_high",
            "smooth_times":smooth_times,"min_size":min_size,"sn_height":sn_height,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":grid0.slon,"startlat":grid0.slat,"dlon":grid0.dlon,"dlat":grid0.dlat,
            "data":data,"output_dir_root":output_dir_root}
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    graphy = json.loads(str_json)
    return graphy




def tran_graphy_to_df(graphy):
    '''

    :param graphy:
    :return:
    '''
    if graphy is None: return None,None

    dtime = int(graphy["dtime"])

    time  = meteva.base.all_type_time_to_time64(graphy["time"])

    level = graphy["level"]

    ids = graphy["features"].keys()

    list1 = []
    for id in ids:
        center = graphy["features"][id]
        lon = center["centrePoint"]["lon"]
        lat = center["centrePoint"]["lat"]
        center_value = center["centrePoint"]["value"]
        area = center["features"]["area"]
        strenght = center["features"]["strenght"]
        dict1 = {"level":level,"time":time,"dtime":dtime,"id" :int(id),"lon":lon,"lat":lat,"value":center_value,"area":area,"strenght":strenght}
        list1.append(dict1)
    center = pd.DataFrame(list1)

    list1 = []
    for id in ids:
        points = graphy["features"][id]["axes"]["point"]
        if len(points)>0:
            array = np.array(points)
            lon = array[:,0]
            lat = array[:,1]
            df = pd.DataFrame({"lon":lon,"lat":lat})
            df["id"] = int(id)
            list1.append(df)
    if len(list1)>0:
        df = pd.concat(list1,axis = 0)
        df["level"] = level
        df["time"] = time
        df["dtime"] = dtime
        df["data0"] = 0
        axes = meteva.base.sta_data(df)
        return center,axes
    else:
        return center,None


from meteva.base import  IV
#计算晴雨预报相对技巧
def pc_skill(pc,pc_base):
    '''

    :param pc:
    :param pc_base:
    :return:
    '''
    if pc_base == 1:
        if pc <1:
            return -IV
        else:
            return 0
    else:
        spc1 = (pc - pc_base) / (1 - pc_base)
        return spc1

def spc(pc,pc_base):
    '''

    :param pc: 省台的正确率
    :param pc_base: 中央台的正确率
    :return: 省台相对中央台的预报技巧
    '''
    if pc_base == 1:
        if pc <1:
            return -IV
        else:
            return 0
    else:
        skill = (pc - pc_base) / (1 - pc_base)
        return round(skill,3)

#计算Ts评分相对技巧
def ts_skill(ts,ts_base):
    '''

    :param ts:
    :param ts_base:
    :return:
    '''
    if ts_base == 1:
        if ts < 1:
            return -IV
        else:
            return 0
    else:
        sts1 = (ts - ts_base) / (1 - ts_base)
        return sts1

def sts(ts,ts_base,nb):
    '''

    :param ts: 省台的ts评分
    :param ts_base: 中央台的ts评分
    :return: 省台相对中央台的预报技巧
    '''
    if ts_base == 1:
        if ts < 1:
            return -IV
        else:
            return 0
    elif ts_base == IV:
        if ts ==IV:
            return 0
        else:
            return 1- 1/nb
    elif ts_base==0 and ts == IV:
        return 1/nb
    else:
        skill = (ts - ts_base) / (1 - ts_base)
        return round(skill,3)

def bias_extend_skill(be,be_base):
    '''

    :param be:
    :param be_base:
    :return:
    '''
    if be_base ==0:
        if be == 0:
            return 0
        else:
            return IV
    else:
        sbi1 = (be_base - be) / be_base
        return sbi1

def sbi(be,be_base):
    '''

    :param be: 省台的偏差幅度
    :param be_base: 中央台的偏差幅度
    :return: 省台相对于中央台的预报技巧
    '''
    if be_base ==0:
        if be == 0:
            return 0
        else:
            return IV
    else:
        skill = (be_base - be) / be_base
        return round(skill,3)

def sfa(far,far_base):
    '''

    :param far: 省台空报率
    :param far_base: 中央台空报率
    :return: 省台相对于中央台的预报技巧
    '''
    if far_base == 0:
        if far ==0:
            return 0
        else:
            return -IV
    else:
        skill = (far_base - far)/far_base
        return round(skill,3)


def spo(pod,pod_base):
    '''

    :param pod: 省命中率
    :param pod_base: 中央台命中率
    :return: 省台相对于中央台的预报技巧
    '''
    if pod_base == 1:
        if pod ==1:
            return 0
        else:
            return -IV
    elif pod_base == IV and pod == IV:
        return 0
    else:
        skill= (pod - pod_base) / (1 - pod_base)
        return round(skill,3)



import numpy as np
import matplotlib.pyplot as plt
import copy
import meteva



def frequency_histogram(ob, fo,grade_list=None, member_list=None,  vmax = None,save_path=None,show = False,dpi = 300,plot = "bar", title="频率统计图",
                        sup_fontsize = 10,width = None,height = None,log_y = False,xlabel = "类别",ylabel = "样本占比"):
    '''
    frequency_histogram 对比测试数据和实况数据的发生的频率
    :param ob: 实况数据 任意维numpy数组
    :param fo: 预测数据 任意维numpy数组,Fo.shape 和Ob.shape一致
    :param grade_list: 如果该参数为None，观测或预报值出现过的值都作为分类标记.
    如果该参数不为None，它必须是一个从小到大排列的实数，以其中列出的数值划分出的多个区间作为分类标签。
    对于预报和观测值不为整数的情况，grade_list 不能设置为None。
    :param save_path: 保存地址
    :return: 无
    '''
    Fo_shape = fo.shape
    Ob_shape = ob.shape
    Ob_shpe_list = list(Ob_shape)
    size = len(Ob_shpe_list)
    ind = -size
    Fo_Ob_index = list(Fo_shape[ind:])
    if Fo_Ob_index != Ob_shpe_list:
        print('预报数据和观测数据维度不匹配')
        return
    Ob_shpe_list.insert(0, -1)
    new_Fo_shape = tuple(Ob_shpe_list)
    new_Fo = fo.reshape(new_Fo_shape)
    new_Fo_shape = new_Fo.shape


    legend = ['观测']
    if member_list is None:
        if new_Fo_shape[0] <= 1:
            legend.append('预报')
        else:
            for i in range(new_Fo_shape[0]):
                legend.append('预报' + str(i + 1))
    else:
        legend.extend(member_list)



    result_array = meteva.method.frequency_table(ob, fo, grade_list=grade_list)
    total_count = np.sum(result_array[0,:])
    result_array /= total_count
    if grade_list is not None:
        if len(grade_list) >10:
            axis = ["<\n" + str(round(grade_list[0],6))]
            for index in range(len(grade_list)):
                axis.append(str(round(grade_list[index],6)))
            axis.append(">=\n" + str(round(grade_list[-1],6)))
        else:
            axis = ["<" + str(round(grade_list[0],6))]
            for index in range(len(grade_list) - 1):
                axis.append("[" + str(round(grade_list[index],6)) + "," + str(round(grade_list[index + 1],6)) + ")")
            axis.append(">=" + str(round(grade_list[-1],6)))


    else:
        new_fo = copy.deepcopy(fo).flatten()
        new_ob = copy.deepcopy(ob).flatten()
        fo_list = list(set(new_fo.tolist()))
        fo_list.extend(list(set(new_ob.tolist())))
        axis = list(set(fo_list))

    name_list_dict = {}
    name_list_dict["legend"] = legend
    name_list_dict[xlabel] = axis
    if log_y:
        vmin = None
    else:
        vmin = 0
    if plot == "bar":
        meteva.base.plot_tools.bar(result_array,name_list_dict,ylabel=ylabel,vmin = vmin,vmax = vmax,save_path = save_path,show = show,dpi = dpi,title=title,
                                   width = width,height = height,sup_fontsize= sup_fontsize,log_y = log_y)
    else:
        meteva.base.plot_tools.plot(result_array, name_list_dict, ylabel=ylabel, vmin=vmin, vmax=vmax, save_path=save_path,
                                   show=show, dpi=dpi, title=title,
                                    width = width,height = height,sup_fontsize= sup_fontsize,log_y = log_y)


def performance_grade(ob, fo, grade_list=None, member_list=None, x_y="sr_pod", save_path=None, show=False,
                dpi=300, title="综合表现图",
                sup_fontsize=10, width=None, height=None):
    '''
    分级降水的综合评分图
    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''


    hfmc_array = meteva.method.multi_category.score.hfmc_grade(ob, fo, grade_list)
    pod = meteva.method.yes_or_no.score.pod_hfmc(hfmc_array)
    sr = meteva.method.yes_or_no.score.sr_hfmc(hfmc_array)

    leftw = 0.6
    rightw = 2
    uphight = 1.2
    lowhight = 1.2
    axis_size_x = 3.7
    axis_size_y = 3.5
    if width is None:
        width = axis_size_x + leftw + rightw

    if height is None:
        height = axis_size_y + uphight + lowhight

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    ax1 = fig.add_axes([leftw / width, lowhight / width, axis_size_x / width, axis_size_y / height])

    x = np.arange(0.0001, 1, 0.0001)
    bias_list = [0.2, 0.4, 0.6, 0.8, 1, 1.25, 1.67, 2.5, 5]
    ts_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
    for i in range(len(bias_list)):
        bias = bias_list[i]
        y1 = bias * x
        x2 = x[y1 < 1]
        y2 = y1[y1 < 1]
        if bias < 1:
            # bias <1 的 线
            ax1.plot(x2, y2, '--', color='k', linewidth=0.5)
            ax1.text(1.01, bias, "bias=" + str(bias), fontsize=sup_fontsize * 0.8)
        elif bias > 1:
            # bias》1的线
            ax1.plot(x2, y2, '--', color='k', linewidth=0.5)
            ax1.text(1.0 / bias - 0.05, 1.02, "bias=" + str(bias), fontsize=sup_fontsize * 0.8)
        else:
            # bias ==1 的线
            ax1.plot(x2, y2, '-', color='k', linewidth=0.5)

    for i in range(len(ts_list)):
        ts = ts_list[i]
        hf = 1
        x2 = np.arange(ts, 1, 0.001)
        hit = hf * x2
        hfm = hit / ts
        m = hfm - hf
        y2 = hit / (hit + m)
        # ts 的线
        plt.plot(x2, y2, "--", color="y", linewidth=0.5)
        error = np.abs(y2 - x2)
        index = np.argmin(error)
        sx = x2[index] + 0.02
        sy = y2[index] - 0.02
        ax1.text(sx, sy, "ts=" + str(ts))

    new_sr = sr.reshape((-1, len(grade_list)+1))
    new_pod = pod.reshape((-1, len(grade_list)+1))

    new_sr_shape = new_sr.shape
    label = []
    legend_num = new_sr_shape[0]
    if member_list is None:
        if legend_num == 1:
            label.append('预报')
        else:
            for i in range(legend_num):
                label.append('预报' + str(i + 1))
    else:
        label.extend(member_list)

    colors = meteva.base.color_tools.get_color_list(legend_num)

    marker = ['o', 'v', 's', 'p', "P", "*", 'h', "X", "d", "1", "+", "x", ".", "^", "<", ">",
              "2", "3", "4", "8", "H", "D", "|", "_"]

    a_list = []
    grade_num = len(grade_list)+1
    grade_legend_strs = ["<" + str(grade_list[0])]
    for i in range(len(grade_list)-1):
        grade_legend_strs.append("[" + str(grade_list[i])+"," + str(grade_list[i+1])+")")
    grade_legend_strs.append(">=" + str(grade_list[-1]))


    if legend_num > 1 and grade_num > 1:
        for line in range(legend_num):
            for i in range(grade_num):
                ax1.plot(new_sr[line, i], new_pod[line, i], marker[i], label=i * line, color=colors[line], markersize=6)
                a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()
        legend2 = ax1.legend(lines[0:len(lines):grade_num], label, loc="upper right",
                             bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)

        legend1 = ax1.legend(lines[:grade_num], grade_legend_strs, loc="lower right",
                             bbox_to_anchor=(1.5, 0), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend1)
        ax1.add_artist(legend2)

    elif legend_num > 1:
        for line in range(legend_num):
            i = 0
            ax1.plot(new_sr[line, i], new_pod[line, i], marker[line], label=i * line, color=colors[line], markersize=6)
            a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()

        legend2 = ax1.legend(lines[0:len(lines):len(grade_list)], label, loc="upper right",
                             bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend2)

    elif grade_num > 1:
        colors = meteva.base.color_tools.get_color_list(grade_num)
        for i in range(grade_num):
            line = 0
            ax1.plot(new_sr[line, i], new_pod[line, i], marker[i], label=i * line, color=colors[i], markersize=6)
            a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()

        legend1 = ax1.legend(lines[:len(grade_list)], grade_legend_strs, loc="upper right",
                             bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend1)

    ax1.set_xlim(0, 1)
    ax1.set_ylim(0, 1)
    if x_y == "sr_pod":
        ax1.set_xlabel("成功率", fontsize=sup_fontsize * 0.9)
        ax1.set_ylabel("命中率", fontsize=sup_fontsize * 0.9)
    else:
        ax1.set_xlabel("空报率", fontsize=sup_fontsize * 0.9)
        ax1.set_ylabel("漏报率", fontsize=sup_fontsize * 0.9)
        x = np.arange(0, 1.01, 0.2)
        ax1.set_xticks(x)
        ax1.set_xticklabels(np.round(1 - x, 1))
        y = np.arange(0, 1.01, 0.2)
        ax1.set_yticks(y)
        ax1.set_yticklabels(np.round(1 - y, 1))

    title = title + "\n"
    ax1.set_title(title, fontsize=sup_fontsize)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()



def performance_multi(ob, fo, grade_list=None, member_list=None, x_y="sr_pod", save_path=None, show=False,
                dpi=300, title="综合表现图",
                sup_fontsize=10, width=None, height=None):
    '''
    分级降水的综合评分图
    :param ob:
    :param fo:
    :param grade_list:
    :return:
    '''


    hfmc_array = meteva.method.multi_category.score.hfmc_multi(ob, fo, grade_list)
    pod = meteva.method.yes_or_no.score.pod_hfmc(hfmc_array)
    sr = meteva.method.yes_or_no.score.sr_hfmc(hfmc_array)

    leftw = 0.6
    rightw = 2
    uphight = 1.2
    lowhight = 1.2
    axis_size_x = 3.7
    axis_size_y = 3.5
    if width is None:
        width = axis_size_x + leftw + rightw

    if height is None:
        height = axis_size_y + uphight + lowhight

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    ax1 = fig.add_axes([leftw / width, lowhight / width, axis_size_x / width, axis_size_y / height])

    x = np.arange(0.0001, 1, 0.0001)
    bias_list = [0.2, 0.4, 0.6, 0.8, 1, 1.25, 1.67, 2.5, 5]
    ts_list = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9]
    for i in range(len(bias_list)):
        bias = bias_list[i]
        y1 = bias * x
        x2 = x[y1 < 1]
        y2 = y1[y1 < 1]
        if bias < 1:
            # bias <1 的 线
            ax1.plot(x2, y2, '--', color='k', linewidth=0.5)
            ax1.text(1.01, bias, "bias=" + str(bias), fontsize=sup_fontsize * 0.8)
        elif bias > 1:
            # bias》1的线
            ax1.plot(x2, y2, '--', color='k', linewidth=0.5)
            ax1.text(1.0 / bias - 0.05, 1.02, "bias=" + str(bias), fontsize=sup_fontsize * 0.8)
        else:
            # bias ==1 的线
            ax1.plot(x2, y2, '-', color='k', linewidth=0.5)

    for i in range(len(ts_list)):
        ts = ts_list[i]
        hf = 1
        x2 = np.arange(ts, 1, 0.001)
        hit = hf * x2
        hfm = hit / ts
        m = hfm - hf
        y2 = hit / (hit + m)
        # ts 的线
        plt.plot(x2, y2, "--", color="y", linewidth=0.5)
        error = np.abs(y2 - x2)
        index = np.argmin(error)
        sx = x2[index] + 0.02
        sy = y2[index] - 0.02
        ax1.text(sx, sy, "ts=" + str(ts))

    new_sr = sr.reshape((-1, len(grade_list)+1))
    new_pod = pod.reshape((-1, len(grade_list)+1))

    new_sr_shape = new_sr.shape
    label = []
    legend_num = new_sr_shape[0]
    if member_list is None:
        if legend_num == 1:
            label.append('预报')
        else:
            for i in range(legend_num):
                label.append('预报' + str(i + 1))
    else:
        label.extend(member_list)

    colors = meteva.base.color_tools.get_color_list(legend_num)

    marker = ['o', 'v', 's', 'p', "P", "*", 'h', "X", "d", "1", "+", "x", ".", "^", "<", ">",
              "2", "3", "4", "8", "H", "D", "|", "_"]

    a_list = []
    grade_num = len(grade_list)+1
    grade_legend_strs = ["<" + str(grade_list[0])]
    for i in range(len(grade_list)-1):
        grade_legend_strs.append("[" + str(grade_list[i])+"," + str(grade_list[i+1])+")")

    grade_legend_strs.append(">=" + str(grade_list[-1]))


    if legend_num > 1 and grade_num > 1:
        for line in range(legend_num):
            for i in range(grade_num):
                ax1.plot(new_sr[line, i], new_pod[line, i], marker[i], label=i * line, color=colors[line], markersize=6)
                a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()
        legend2 = ax1.legend(lines[0:len(lines):grade_num], label, loc="upper right",
                             bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)

        legend1 = ax1.legend(lines[:grade_num], grade_legend_strs, loc="lower right",
                             bbox_to_anchor=(1.5, 0), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend1)
        ax1.add_artist(legend2)

    elif legend_num > 1:
        for line in range(legend_num):
            i = 0
            ax1.plot(new_sr[line, i], new_pod[line, i], marker[line], label=i * line, color=colors[line], markersize=6)
            a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()

        legend2 = ax1.legend(lines[0:len(lines):len(grade_list)], label, loc="upper right",
                             bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend2)

    elif grade_num > 1:
        colors = meteva.base.color_tools.get_color_list(grade_num)
        for i in range(grade_num):
            line = 0
            ax1.plot(new_sr[line, i], new_pod[line, i], marker[i], label=i * line, color=colors[i], markersize=6)
            a_list.append(i * line)
        lines, label1 = ax1.get_legend_handles_labels()

        legend1 = ax1.legend(lines[:len(grade_list)], grade_legend_strs, loc="upper right",
                             bbox_to_anchor=(1.5, 1), ncol=1, fontsize=sup_fontsize * 0.9)
        ax1.add_artist(legend1)

    ax1.set_xlim(0, 1)
    ax1.set_ylim(0, 1)
    if x_y == "sr_pod":
        ax1.set_xlabel("成功率", fontsize=sup_fontsize * 0.9)
        ax1.set_ylabel("命中率", fontsize=sup_fontsize * 0.9)
    else:
        ax1.set_xlabel("空报率", fontsize=sup_fontsize * 0.9)
        ax1.set_ylabel("漏报率", fontsize=sup_fontsize * 0.9)
        x = np.arange(0, 1.01, 0.2)
        ax1.set_xticks(x)
        ax1.set_xticklabels(np.round(1 - x, 1))
        y = np.arange(0, 1.01, 0.2)
        ax1.set_yticks(y)
        ax1.set_yticklabels(np.round(1 - y, 1))

    title = title + "\n"
    ax1.set_title(title, fontsize=sup_fontsize)
    if save_path is None:
        show = True
    else:
        plt.savefig(save_path, bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show is True:
        plt.show()
    plt.close()


import pandas as pd
import copy
import meteva
import datetime
import math
import numpy as np

def concat(data_list):
    data1_list = []
    for data in data_list:
        if data is not None:
            data1_list.append(data)
    if len(data1_list) == 0:
        print("无需要拼接的数据")
        return

    if isinstance(data1_list[0], pd.DataFrame):
        sta = pd.concat(data1_list,axis=0)
        sta.drop_duplicates(keep="first",inplace=True)
        sta.attrs = copy.deepcopy(data1_list[0].attrs)
        #sta[["dtime"]] = sta[['dtime']].astype(int)
        return sta
    else:
        ngrd = len(data1_list)
        grid_combined = None
        for i in range(ngrd):
            grid1 = meteva.base.get_grid_of_data(data1_list[i])
            grid_combined = meteva.base.get_outer_grid(grid_combined, grid1, used_coords="all")
        grd_all = meteva.base.grid_data(grid_combined)
        grid_xy = meteva.base.grid(grid_combined.glon, grid_combined.glat)
        grd_all.values[:, :, :, :, :, :] = meteva.base.IV
        levels_all = grd_all["level"].values
        times_all = grd_all["time"].values
        dtimes_all = grd_all["dtime"].values
        members_all = grd_all["member"].values
        index_level = {}
        for i in range(len(levels_all)):
            index_level[levels_all[i]] = i
        index_time = {}
        for i in range(len(times_all)):
            index_time[times_all[i]] = i
        index_dtime = {}
        for i in range(len(dtimes_all)):
            index_dtime[dtimes_all[i]] = i
        index_member = {}
        for i in range(len(members_all)):
            index_member[members_all[i]] = i

        for i in range(ngrd):
            grd_interp = meteva.base.interp_gg_linear(data1_list[i], grid_xy)
            levels = grd_interp["level"].values
            times = grd_interp["time"].values
            dtimes = grd_interp["dtime"].values
            members = grd_interp["member"].values
            for i in range(len(levels)):
                i_level = index_level[levels[i]]
                for j in range(len(times)):
                    i_time = index_time[times[j]]
                    for k in range(len(dtimes)):
                        i_dtime = index_dtime[dtimes[k]]
                        for m in range(len(members)):
                            i_member = index_member[members[m]]
                            grd_all.values[i_member, i_level, i_time, i_dtime, :, :] = grd_interp.values[m, i, j, k, :,
                                                                                       :]
        return grd_all

# 两个站点信息合并为一个，在原有的dataframe的基础上增加行数
def combine_join(sta, sta1):
    if (sta is None):
        return sta1
    elif (sta1 is None):
        return sta
    else:
        data_name1 = meteva.base.basicdata.get_stadata_names(sta)
        data_name2 = meteva.base.basicdata.get_stadata_names(sta1)
        if data_name1 == data_name2:
            sta = concat([sta, sta1])
        else:
            sta2 = copy.deepcopy(sta1)
            meteva.base.basicdata.set_stadata_names(sta2,data_name1)
            sta = concat([sta, sta2])
    sta = sta.reset_index(drop=True)
    return sta


# 两个站点信息合并为一个，以站号为公共部分，在原有的dataframe的基础上增加列数
def combine_on_id(sta, sta1,how = "inner"):
    if sta is None:
        return sta1
    elif sta1 is None:
        return sta
    else:
        df = pd.merge(sta, sta1, on='id', how=how)
        columns = list(sta.columns)
        len_sta = len(columns)
        # 删除合并后第二组时空坐标信息
        drop_col = list(df.columns[len_sta:len_sta + 5])
        df.drop(drop_col, axis=1, inplace=True)
        columns_dim = list(sta.columns)[0:6]
        columns_data = list(df.columns)[6:]
        columns = columns_dim + columns_data
        df.columns = columns
        return df

def that_the_name_exists(list, value):
    '''
    that_the_name_exists判断value是否在list中  如果存在改value直到不在list中为止
    :param list: 一个要素名列表
    :param value:  要素名
    :return:
    '''
    value = str(value)
    list = [str(i) for i in list]
    if value in list:
        value = str(value) + 'x'
        return that_the_name_exists(list, value)
    else:
        return value

#  两个站点信息合并为一个，以站号为公共部分，在原有的dataframe的基础上增加列数
def combine_on_all_coords(sta, sta1,how = "inner"):
    '''
    merge_on_all_dim 合并两个sta_dataframe并且使要素名不重复
    :param sta: 一个站点dataframe
    :param sta1: 一个站点dataframe
    :return:
    '''
    if (sta is None):
        return sta1
    elif sta1 is None:
        return sta
    else:
        columns = ['level', 'time', 'dtime', 'id', 'lon', 'lat']
        sta_value_columns = sta.iloc[:, 6:].columns.values.tolist()
        sta1_value_columns = sta1.iloc[:, 6:].columns.values.tolist()
        if len(sta_value_columns) >= len(sta1_value_columns):

            for sta1_value_column in sta1_value_columns:
                ago_name = copy.deepcopy(sta1_value_column)
                sta1_value_column = that_the_name_exists(sta_value_columns, sta1_value_column)
                sta1.rename(columns={ago_name: sta1_value_column},inplace=True)
        else:
            for sta_value_column in sta_value_columns:
                ago_name = copy.deepcopy(sta_value_column)
                sta_value_column = that_the_name_exists(sta1_value_columns, sta_value_column)
                sta.rename(columns={ago_name: sta_value_column})
        df = pd.merge(sta, sta1, on=columns, how=how)
        df.attrs = copy.deepcopy(sta.attrs)
        return df

def combine_on_leve_time_id(sta,sta1):
    '''
    merge_on_all_dim 合并两个sta_dataframe并且使要素名不重复
    :param sta: 一个站点dataframe
    :param sta1: 一个站点dataframe
    :return:
    '''
    if (sta is None):
        return sta1
    elif sta1 is None:
        return sta
    else:
        columns = ['level', 'time',  'id']
        sta_value_columns = sta.iloc[:, 6:].columns.values.tolist()
        sta2 = copy.deepcopy(sta1)
        sta2_value_columns = sta2.iloc[:, 6:].columns.values.tolist()

        if len(sta_value_columns) >= len(sta2_value_columns):
            for sta2_value_column in sta2_value_columns:
                ago_name = copy.deepcopy(sta2_value_column)
                sta2_value_column = that_the_name_exists(sta_value_columns, sta2_value_column)
                sta2.rename(columns={ago_name: sta2_value_column},inplace=True)
        else:
            for sta_value_column in sta_value_columns:
                ago_name = copy.deepcopy(sta_value_column)
                sta_value_column = that_the_name_exists(sta2_value_columns, sta_value_column)
                sta.rename(columns={ago_name: sta_value_column})
        sta2.drop(['dtime',"lon","lat"], axis=1, inplace=True)
        df = pd.merge(sta, sta2, on=columns, how='inner')
        return df

def combine_on_level_time_dtime_id(sta, sta1,how = 'inner'):
    '''
    merge_on_all_dim 合并两个sta_dataframe并且使要素名不重复
    :param sta: 一个站点dataframe
    :param sta1: 一个站点dataframe
    :return:
    '''

    if (sta is None):
        return sta1
    elif sta1 is None:
        return sta
    else:
        columns = ['level', 'time', 'dtime', 'id']
        sta_value_columns = sta.iloc[:, 6:].columns.values.tolist()
        sta2 = copy.deepcopy(sta1)
        sta2_value_columns = sta2.iloc[:, 6:].columns.values.tolist()

        if len(sta_value_columns) >= len(sta2_value_columns):
            for sta2_value_column in sta2_value_columns:
                ago_name = copy.deepcopy(sta2_value_column)
                sta2_value_column = that_the_name_exists(sta_value_columns, sta2_value_column)
                sta2.rename(columns={ago_name: sta2_value_column},inplace=True)
        else:
            for sta_value_column in sta_value_columns:
                ago_name = copy.deepcopy(sta_value_column)
                sta_value_column = that_the_name_exists(sta2_value_columns, sta_value_column)
                sta.rename(columns={ago_name: sta_value_column})
        if(how == "inner"):
            sta2.drop(["lon","lat"], axis=1, inplace=True)
            df = pd.merge(sta, sta2, on=columns, how=how)
        else:
            sta3 = sta.copy()
            #sta3.drop(["lon","lat"], axis=1, inplace=True)
            df = pd.merge(sta3, sta2, on=columns, how=how)
            index = np.isnan(df.loc[:,"lon_x"])
            if index.size >0:
                df.loc[index,"lon_x"] = df.loc[index,"lon_y"]
                df.loc[index,"lat_x"] = df.loc[index,"lat_y"]
            df.drop(["lon_y","lat_y"], axis=1, inplace=True)
            columns_Name = df.columns.values.tolist()
            columns_Name[4] = "lon"
            columns_Name[5] = "lat"
            df.columns = columns_Name
            if(len(df.index) == 0):
                print("no matched line")
                return None
            df = meteva.base.sta_data(df)
        df.attrs = copy.deepcopy(sta.attrs)

        return df


def combine_on_level_time_dtime(sta, sta1,how = 'inner'):
    '''
    merge_on_all_dim 合并两个sta_dataframe并且使要素名不重复
    :param sta: 一个站点dataframe
    :param sta1: 一个站点dataframe
    :return:
    '''
    if (sta is None):
        return sta1
    elif sta1 is None:
        return sta
    else:
        columns = ['level', 'time', 'dtime']
        sta_value_columns = sta.iloc[:, 6:].columns.values.tolist()
        sta2 = copy.deepcopy(sta1)
        sta2_value_columns = sta2.iloc[:, 6:].columns.values.tolist()

        if len(sta_value_columns) >= len(sta2_value_columns):
            for sta2_value_column in sta2_value_columns:
                ago_name = copy.deepcopy(sta2_value_column)
                sta2_value_column = that_the_name_exists(sta_value_columns, sta2_value_column)
                sta2.rename(columns={ago_name: sta2_value_column},inplace=True)
        else:
            for sta_value_column in sta_value_columns:
                ago_name = copy.deepcopy(sta_value_column)
                sta_value_column = that_the_name_exists(sta2_value_columns, sta_value_column)
                sta.rename(columns={ago_name: sta_value_column})
        if(how == "inner"):
            sta2.drop(["id","lon","lat"], axis=1, inplace=True)
            df = pd.merge(sta, sta2, on=columns, how=how)
        else:
            sta3 = sta.copy()
            sta3.drop(["id","lon","lat"], axis=1, inplace=True)
            df = pd.merge(sta3, sta2, on=columns, how=how)
            if(len(df.index) == 0):
                print("no matched line")
                return None
            df = meteva.base.sta_data(df)
        df.attrs = copy.deepcopy(sta.attrs)
        return df


def combine_on_obTime_id(sta_ob,sta_fo_list,need_match_ob = False,how_fo = "inner"):
    '''
    将观测
    :param sta_ob:
    :param sta_fo_list:
    :return:
    '''
    if not isinstance(sta_fo_list, list):
        sta_fo_list = [sta_fo_list]
    #预报时效的时间处理
    dtime_units = "hour"
    for sta_fo1 in sta_fo_list:
        if sta_fo1.attrs is not None:
            if "dtime_units" in sta_fo1.attrs.keys():
            # if len(sta_fo1.attrs) >0:
                if(sta_fo1.attrs["dtime_units"] != "hour"):
                    dtime_units = "minute"
    if dtime_units != "hour":
        for sta_fo1 in sta_fo_list:
            if len(sta_fo1.attrs)  == 0 or sta_fo1.attrs["dtime_units"] == "hour" :
                sta_fo1["dtime"] *= 60
                sta_fo1.attrs["dtime_units"] = "minute"


    dtime_list = list(set(sta_fo_list[0]['dtime'].values.tolist()))
    nsta_ob = len(sta_ob.index)
    if(nsta_ob * len(dtime_list) >= 10000000):
        if nsta_ob >= 10000000:
            print("请注意，在大规模数据匹配合并时，need_match_ob 参数将自动切换为True")
            return combine_on_obTime_id_bigData(sta_ob,sta_fo_list,how_fo = how_fo)
        else:
            return combine_on_obTime_id_bigData(sta_ob, sta_fo_list,need_match_ob=need_match_ob,g = "dtime",how_fo= how_fo)
    else:


        sta_combine_fo = None
        for sta_fo in sta_fo_list:
            sta_combine_fo = combine_on_level_time_dtime_id(sta_combine_fo, sta_fo,how = how_fo)
            if sta_combine_fo is not None:
                sta_combine_fo = sta_combine_fo.fillna(meteva.base.IV)


        if sta_ob is None:
            sta_combine = None
        else:
            dtime_list = list(set(sta_combine_fo['dtime'].values.tolist()))
            sta_combine = []
            for dtime in dtime_list:
                sta = copy.deepcopy(sta_ob)
                if dtime_units == "hour":
                    sta["time"] = sta["time"] - datetime.timedelta(hours= dtime)
                else:
                    sta["time"] = sta["time"] - datetime.timedelta(minutes=dtime)
                sta["dtime"] = dtime
                sta_combine.append(sta)
            sta_combine = concat(sta_combine)


        if need_match_ob:
            sta_combine = meteva.base.not_IV(sta_combine)
            sta_combine = combine_on_level_time_dtime_id(sta_combine, sta_combine_fo, how="inner")
        else:
            sta_combine = combine_on_level_time_dtime_id(sta_combine,sta_combine_fo,how="right")
            if sta_combine is not None:
                sta_combine = sta_combine.fillna(meteva.base.IV)

        if sta_combine is None:
            print("观测和预报数据未能实现匹配，请检查观测和预报的层次、时间、时效、站号是否有严格的匹配关系")
        meteva.base.set_stadata_attrs(sta_combine,dtime_units=dtime_units)
        sta_combine.attrs = copy.deepcopy(sta_ob.attrs)
        sta_combine.drop_duplicates(subset=["level", "time", "dtime", "id"], inplace=True)
        sta_combine.sort_values(by=["level", "time", "dtime", "id"], inplace=True)

        return sta_combine


def combine_on_obTime_one_id(sta_ob,sta_fo_list,how = "inner",how_fo = "inner"):
    '''
    将观测
    :param sta_ob:
    :param sta_fo_list:
    :return:
    '''
    if not isinstance(sta_fo_list, list):
        sta_fo_list = [sta_fo_list]

    dtime_units ="hour"
    if len(sta_fo_list[0].attrs) >0:
        if "dtime_units" in sta_fo_list[0].attrs:
            if sta_fo_list[0].attrs["dtime_units"] != "hour":
                dtime_units = "minute"


    sta_combine_fo = None
    for sta_fo in sta_fo_list:
        sta_combine_fo = combine_on_level_time_dtime(sta_combine_fo, sta_fo,how= how_fo)

    if sta_ob is None:
        sta_combine = None
    else:
        dtime_list = list(set(sta_combine_fo['dtime'].values.tolist()))
        #print(dtime_list)
        sta_combine = []
        for dtime in dtime_list:
            sta = copy.deepcopy(sta_ob)
            if dtime_units == "hour":
                sta["time"] = sta["time"] - datetime.timedelta(hours= dtime)
            else:
                sta["time"] = sta["time"] - datetime.timedelta(minutes=dtime)
            sta["dtime"] = dtime
            sta_combine.append(sta)
        sta_combine = concat(sta_combine)

    sta_combine = combine_on_level_time_dtime(sta_combine, sta_combine_fo,how = how)
    if sta_combine is not None:
        sta_combine = sta_combine.fillna(meteva.base.IV)
    sta_combine.attrs = copy.deepcopy(sta_ob.attrs)
    return sta_combine

def combine_on_obTime_id_bigData(sta_ob,sta_fo_list,need_match_ob = True,g = "id",how_fo = None):
    import sys,gc
    '''
    将观测
    :param sta_ob:
    :param sta_fo_list:
    :return:
    '''
    if not isinstance(sta_fo_list, list):
        print("the second args shold be a list")
        return
    sta_all = None

    if g =="id":
        grouped_ob = dict(list(sta_ob.groupby("id")))
        nfo = len(sta_fo_list)
        grouped_fo_list=[]
        for i in range(nfo):
            grouped_fo_list.append(dict(list(sta_fo_list[i].groupby("id"))))
        id_ob = list(grouped_ob.keys())
        sys._clear_type_cache()
        gc.collect()
        sta_all = []
        n_id = len(id_ob)

        if need_match_ob:
            how = "inner"
        else:
            how = "right"
        for i in range(n_id):
            rate = int((i/n_id)*100)
            if rate%5 == 0:
                if abs(i - rate * 0.01 * n_id)<1:
                    print(str(rate) + "% combined")

            key = id_ob[i]
            all_fos_have = True
            sta_ob_one_id = grouped_ob.pop(key)
            sta_fos_one_id = []
            for i in range(nfo):
                if key in grouped_fo_list[i].keys():
                   sta_fos_one_id.append(grouped_fo_list[i].pop(key))
                else:
                    all_fos_have = False
            if all_fos_have:
                combine_one = combine_on_obTime_one_id(sta_ob_one_id,sta_fos_one_id,how = how,how_fo = how_fo)
                sta_all.append(combine_one)
        sta_all = concat(sta_all)
    elif g == "dtime":
        nfo = len(sta_fo_list)
        grouped_fo_list = []
        dtime_list = []
        for i in range(nfo):
            dict1 = dict(list(sta_fo_list[i].groupby("dtime")))
            grouped_fo_list.append(dict1)
            dtime_list.extend(list(dict1.keys()))
        dtime_list = list(set(dtime_list))
        sys._clear_type_cache()
        gc.collect()
        sta_list = []
        n_dtime = len(dtime_list)

        for i in range(n_dtime):
            rate = int((i/n_dtime)*100)
            print(str(rate) + "% combined")

            key = dtime_list[i]
            all_fos_have = True
            sta_fos_one_dtime= []

            for i in range(nfo):
                if key in grouped_fo_list[i].keys():
                    sta_fos_one_dtime.append(grouped_fo_list[i].pop(key))
                else:
                    if how_fo == "outer":
                        pass
                    else:
                        all_fos_have = False
            if all_fos_have:
                combine_one = combine_on_obTime_id(sta_ob, sta_fos_one_dtime, need_match_ob=need_match_ob,how_fo=how_fo)
                sta_list.append(combine_one)
                del sta_fos_one_dtime

        sta_all = concat(sta_list)
        del sta_list
    if sta_all is not None:
        sta_all = sta_all.fillna(meteva.base.IV)
    sta_all.attrs = copy.deepcopy(sta_ob.attrs)
    sta_all.drop_duplicates(subset=["level","time","dtime","id"],inplace=True)
    sta_all.sort_values(by=["level", "time", "dtime", "id"], inplace=True)

    names = meteva.base.get_stadata_names(sta_ob)
    for i in range(len(sta_fo_list)):
        names.extend(meteva.base.get_stadata_names(sta_fo_list[i]))
    sta_all = meteva.base.in_member_list(sta_all,member_list=names)

    return sta_all


def combine_on_obTime(sta_ob,sta_fo_list,need_match_ob = False):
    if not isinstance(sta_fo_list, list):
        sta_fo_list = [sta_fo_list]

    dtime_list = list(set(sta_fo_list[0]['dtime'].values.tolist()))
    sta_combine = []
    for dtime in dtime_list:
        sta = sta_ob.copy()
        sta["time"] = sta["time"] - datetime.timedelta(hours= dtime)
        sta["dtime"] = dtime
        sta_combine.append(sta)
    sta_combine = concat(sta_combine)

    sta_combine_fo = None
    for sta_fo in sta_fo_list:
        sta_combine_fo = combine_on_all_coords(sta_combine_fo, sta_fo)


    if need_match_ob:
        sta_combine = combine_on_all_coords(sta_combine, sta_combine_fo, how="inner")
    else:
        sta_combine = combine_on_all_coords(sta_combine,sta_combine_fo,how="right")
        sta_combine = sta_combine.fillna(meteva.base.IV)

    sta_combine.attrs = copy.deepcopy(sta_ob.attrs)
    return sta_combine

def combine_on_bak_idandobTime1(sta_list):
    '''
    merge_on_id_and_obTime  合并多个sta——dataframe  并且保证合并后的dataframe要素名不重复
    :param sta_list:   含有多个sta_dataframe的列表
    :return:
    '''
    intersection_of_data = None
    for sta in sta_list:
        sta['dtime'] = sta['dtime'].map(lambda x: datetime.timedelta(hours=x))
        sta['time'] = sta['time'] + sta['dtime']
        sta['dtime'] = 0

        intersection_of_data = combine_on_all_coords(intersection_of_data, sta)
    intersection_of_data.attrs = copy.deepcopy(sta_list[0].attrs)
    return intersection_of_data




def combine_expand_IV(sta,sta_with_IV):
    '''
        将观测
        :param sta_ob:
        :param sta_fo_list:
        :return:
        '''

    #sta_with_IV1 = meteva.base.sta_data(sta_with_IV)
    sta_with_IV1 = copy.deepcopy(sta_with_IV)
    columns = ["level","time","dtime","id"]
    for i in range(4):
        sta_expand = []
        if sta_with_IV.iloc[0,i] == meteva.base.IV or pd.isnull(sta_with_IV.iloc[0,i]):
            value_list = list(set(sta.iloc[:,i].values.tolist()))
            if i == 1:
                for j in range(len(value_list)):
                    value_list[j] = meteva.base.tool.all_type_time_to_time64(value_list[j])
            for value in value_list:
                sta1 = copy.deepcopy(sta_with_IV1)
                sta1.iloc[:,i] = value
                sta_expand.append(sta1)
            sta_with_IV1 = concat(sta_expand)
    #sta_with_IV1 = sta_with_IV1.dropna()
    sta_combine = combine_on_level_time_dtime_id(sta, sta_with_IV1)
    return sta_combine

def combine_expand(sta,sta_type):
    sta_type1 = meteva.base.sta_data(sta_type)
    return combine_expand_IV(sta,sta_type1)

def get_inner_grid(grid0,grid1,used_coords = "xy"):
    if used_coords =="xy":
        si = 0
        sj = 0
        ei = 0
        ej = 0
        if(grid1.slon > grid0.slon):
            si = int(math.ceil((grid1.slon - grid0.slon)/grid0.dlon))
        if(grid1.slat > grid0.slat):
            sj = int(math.ceil((grid1.slat - grid0.slat)/grid0.dlat))
        if(grid1.elon < grid0.elon):
            ei = int(math.ceil((grid0.elon - grid1.elon)/grid0.dlon))
        if(grid1.elat < grid0.elat):
            ej = int(math.ceil((grid0.elat - grid1.elat)/grid0.dlat))
        slon = grid0.slon + si * grid0.dlon
        slat = grid0.slat + sj * grid0.dlat
        elon = grid0.elon - ei * grid0.dlon
        elat = grid0.elat - ej * grid0.dlat
        grid_inner = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat],grid0.gtime,grid0.dtimes,grid0.levels,grid0.members)
        return grid_inner
    elif used_coords == "all":
        if grid0 is None:return grid1
        if grid1 is None:return grid0
        si = 0
        sj = 0
        ei = 0
        ej = 0
        if(grid1.slon > grid0.slon):
            si = int(math.ceil((grid1.slon - grid0.slon)/grid0.dlon))
        if(grid1.slat > grid0.slat):
            sj = int(math.ceil((grid1.slat - grid0.slat)/grid0.dlat))
        if(grid1.elon < grid0.elon):
            ei = int(math.ceil((grid0.elon - grid1.elon)/grid0.dlon))
        if(grid1.elat < grid0.elat):
            ej = int(math.ceil((grid0.elat - grid1.elat)/grid0.dlat))


        slon = grid0.slon + si * grid0.dlon
        slat = grid0.slat + sj * grid0.dlat
        elon = grid0.elon - ei * grid0.dlon
        elat = grid0.elat - ej * grid0.dlat

        times0 = pd.date_range(grid0.gtime[0], grid0.gtime[1], freq=grid0.gtime[2])
        times1 = pd.date_range(grid1.gtime[0], grid1.gtime[1], freq=grid1.gtime[2])

        times = list(set(times0) & set(times1))
        times.sort()

        if len(times) == 1:
            gtime_d = "1h"
        else:
            gtime_d = times[1] - times[0]

        dtimes = list(set(grid0.dtimes)&set(grid1.dtimes))
        dtimes.sort()
        levels = list(set(grid0.levels)&set(grid1.levels))
        levels.sort()
        members1= []
        for member in grid0.members:
            members1.append(member)
        for member in grid1.members:
            if member not in members1:
                members1.append(member)

        grid_inner = meteva.base.grid([slon, elon, grid0.dlon], [slat, elat, grid0.dlat], [times[0],times[-1],gtime_d], dtimes,
                                      levels, members1)
        return grid_inner
    else:
        pass

def get_outer_grid(grid0,grid1,used_coords = "xy"):
    if used_coords == "xy":
        slon = min(grid0.slon,grid1.slon)
        slat = min(grid0.slat,grid1.slat)
        elon = max(grid0.elon,grid1.elon)
        elat = max(grid0.elat,grid1.elat)
        grid_outer = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat],grid0.gtime,grid0.dtimes,grid0.levels,grid0.members)
        return grid_outer
    elif used_coords == "all":
        if grid0 is None:return grid1
        if grid1 is None:return grid0
        slon = min(grid0.slon, grid1.slon)
        slat = min(grid0.slat, grid1.slat)
        elon = max(grid0.elon, grid1.elon)
        elat = max(grid0.elat, grid1.elat)

        times0 = pd.date_range(grid0.gtime[0], grid0.gtime[1], freq=grid0.gtime[2])
        times1 = pd.date_range(grid1.gtime[0], grid1.gtime[1], freq=grid1.gtime[2])

        times = list(set(times0)|set(times1))
        times.sort()
        if len(times) == 1:
            gtime_d = "1h"
        else:
            gtime_d = times[1] - times[0]

        dtimes = list(set(grid0.dtimes)|set(grid1.dtimes))
        dtimes.sort()

        levels = list(set(grid0.levels)|set(grid1.levels))
        levels.sort()

        members = copy.deepcopy(grid0.members)
        if not isinstance(members,list):
            members = members.tolist()
        for member in grid1.members:
            if member not in members:
                members.append(member)
        grid_outer = meteva.base.grid([slon, elon, grid0.dlon], [slat, elat, grid0.dlat], [times[0],times[-1],gtime_d], dtimes,
                                      levels, members)
        return grid_outer
    else:
        pass
def expand_to_contain_another_grid(grd0,grid1,used_coords = "xy",outer_value = 0):
    grid0 = meteva.base.get_grid_of_data(grd0)
    grid_outer = get_outer_grid(grid0,grid1,used_coords = used_coords)
    grd1 = meteva.base.grid_data(grid_outer)
    grd1.values[...] = outer_value

    si = 0
    sj = 0
    if (grid1.slon < grid0.slon):
        si = int(round((grid0.slon - grid1.slon) / grid0.dlon))
    if (grid1.slat < grid0.slat):
        sj = int(round((grid0.slat - grid1.slat) / grid0.dlat))
    grd1.values[:,:,:,:,sj:(sj + grid0.nlat), si:(si + grid0.nlon)] = grd0.values[...]
    return grd1


def combine_griddata(griddata_list,dtime_list  = None,how = "inner"):
    '''
    :param griddata_list: 网格数据列表
    :return:
    '''
    # 统计网格信息

    ngrd = len(griddata_list)
    grid_combined = None
    for i in range(ngrd):
        if griddata_list[i] is None: continue
        grid1 = meteva.base.get_grid_of_data(griddata_list[i])
        if how =="inner":
            grid_combined = meteva.base.get_inner_grid(grid_combined,grid1,used_coords="all")
        elif how == "outer":
            grid_combined = meteva.base.get_outer_grid(grid_combined, grid1, used_coords="all")

    #print(grid_combined)
    if grid_combined is None:
        print("没有可合并的格点数据")
    if dtime_list is not None:
        grid_combined.dtimes = dtime_list

    grd_all = meteva.base.grid_data(grid_combined)
    grid_xy = meteva.base.grid(grid_combined.glon,grid_combined.glat)

    #grd_all.values[:,:,:,:,:,:] = meteva.base.IV
    grd_all.values[:, :, :, :, :, :] = np.nan
    levels_all = grd_all["level"].values
    times_all = grd_all["time"].values
    dtimes_all = grd_all["dtime"].values
    members_all = grd_all["member"].values
    index_level = {}
    for i in range(len(levels_all)):
        index_level[levels_all[i]] = i
    index_time = {}
    for i in range(len(times_all)):
        index_time[times_all[i]] = i
    index_dtime = {}
    for i in range(len(dtimes_all)):
        index_dtime[dtimes_all[i]] = i
    index_member = {}
    for i in range(len(members_all)):
        index_member[members_all[i]] = i

    for i in range(ngrd):
        if griddata_list[i] is None:continue
        grd_interp = meteva.base.interp_gg_linear(griddata_list[i],grid_xy)
        levels = grd_interp["level"].values
        times = grd_interp["time"].values
        dtimes = grd_interp["dtime"].values
        members = grd_interp["member"].values
        for i in range(len(levels)):
            if not levels[i] in levels_all:continue
            i_level = index_level[levels[i]]
            for j in range(len(times)):
                if not times[j] in times_all: continue
                i_time = index_time[times[j]]
                for k in range(len(dtimes)):
                    if not dtimes[k] in dtimes_all:continue
                    i_dtime = index_dtime[dtimes[k]]
                    for m in range(len(members)):
                        if not members[m] in members_all:continue
                        i_member = index_member[members[m]]
                        grd_all.values[i_member,i_level,i_time,i_dtime,:,:] =  grd_interp.values[m, i, j, k, :, :]
    return grd_all






import os
from datetime import datetime
import numpy as np

def read_micaps14(filename,time = None,dtime = 0,data_name = None):
    """
    Read micaps 14 file (编辑图象的图元数据, 即交互操作结果数据).
    :param fname: micaps 14 filename.
    :return: data dictionary
    :Examples:
    #>>> data = read_micaps_14("Z:/diamond/update/rr082008.024")
    """

    # check file exist
    if not os.path.isfile(filename):
        return None

    # read contents
    encodings = ['utf-8', 'gb18030', 'GBK']
    txt = None
    for encoding in encodings:
        txt = None
        try:
            with open(filename, 'r', encoding=encoding) as f:
                txt = f.read().replace('\n', ' ').split()
        except Exception:
            pass
    if txt is None:
        print("Micaps 14 file error: " + filename)
        return None

    # head information
    _ = txt[2]

    # date and time

    if time is None:
        year = int(txt[3]) if len(txt[3]) == 4 else int(txt[3]) + 2000
        month = int(txt[4])
        day = int(txt[5])
        hour = int(txt[6])
        time = datetime(year, month, day, hour)
    fhour = int(txt[7])

    # ======================================================
    # read lines
    # ======================================================
    lines = None
    if 'LINES:' in txt:
        # get the start position
        idx = txt.index('LINES:')

        # number of lines
        number = int(txt[idx+1])
        idx += 2

        # loop every line
        if number > 0:
            # define data
            line_width = []
            line_xyz_num = []
            line_xyz = []
            line_label_num = []
            line_label = []
            line_label_xyz = []

            for _ in range(number):
                # line width
                width = float(txt[idx])
                line_width.append(width)
                idx += 1

                # line xyz point number
                xyz_num = int(txt[idx])
                line_xyz_num.append(xyz_num)
                idx += 1

                # line xyz
                xyz = np.array(txt[idx:(idx + 3*xyz_num)]).astype(np.float)
                xyz.shape = [xyz_num, 3]
                line_xyz.append(xyz)
                idx += xyz_num * 3

                # line label
                label = txt[idx]
                line_label.append(label)
                idx += 1

                # line label number
                label_num = int(txt[idx])
                line_label_num.append(label_num)
                idx += 1

                # label xyz
                if label_num > 0:
                    label_xyz = np.array(
                        txt[idx:(idx + 3*label_num)]).astype(np.float)
                    label_xyz.shape = [label_num, 3]
                    line_label_xyz.append(label_xyz)
                    idx += label_num * 3
                else:
                    line_label_xyz.append([])

                # construct line data type
                lines = {
                    "line_width": line_width, "line_xyz_num": line_xyz_num,
                    "line_xyz": line_xyz, "line_label_num": line_label_num,
                    "line_label": line_label, "line_label_xyz": line_label_xyz}

    # ======================================================
    # read line symbols
    # ======================================================
    lines_symbol = None
    if 'LINES_SYMBOL:' in txt:
        # get the start position
        idx = txt.index('LINES_SYMBOL:')

        # number of line symbols
        number = int(txt[idx + 1])
        idx += 2

        # loop every line symbol
        if number > 0:
            # define data
            linesym_code = []
            linesym_width = []
            linesym_xyz_num = []
            linesym_xyz = []

            for _ in range(number):
                # line symbol code
                code = int(txt[idx])
                linesym_code.append(code)
                idx += 1

                # line width
                width = float(txt[idx])
                linesym_width.append(width)
                idx += 1

                # line symbol xyz point number
                xyz_num = int(txt[idx])
                linesym_xyz_num.append(xyz_num)
                idx += 1

                # line symbol xyz
                xyz = np.array(txt[idx:(idx + 3*xyz_num)]).astype(np.float)
                xyz.shape = [xyz_num, 3]
                linesym_xyz.append(xyz)
                idx += xyz_num * 3

                # line symbol label
                label = txt[idx]
                idx += 1

                # line symbol label number
                label_num = int(txt[idx])
                idx += label_num * 3 + 1

            lines_symbol = {"linesym_code": linesym_code,
                            "linesym_width": linesym_width,
                            "linesym_xyz_num": linesym_xyz_num,
                            "linesym_xyz": linesym_xyz}

    # ======================================================
    # read symbol
    # ======================================================
    symbols = None
    if "SYMBOLS:" in txt:
        # start position of symbols
        idx = txt.index("SYMBOLS:")

        # number of lines
        number = int(txt[idx + 1])
        idx += 2

        # loop every symbol
        if number > 0:
            # define data
            symbol_code = []
            symbol_xyz = []
            symbol_value = []

            for _ in range(number):
                # symbol code
                code = int(txt[idx])
                symbol_code.append(code)
                idx += 1

                # symbol xyz
                xyz = np.array(txt[idx:(idx + 3)]).astype(np.float)
                symbol_xyz.append(xyz)
                idx += 3

                # symbol value
                value = txt[idx]
                symbol_value.append(value)
                idx += 1

            symbols = {"symbol_code": symbol_code,
                       "symbol_xyz": symbol_xyz,
                       "symbol_value": symbol_value}

    # ======================================================
    # read closed contours
    # ======================================================
    closed_contours = None
    if "CLOSED_CONTOURS:" in txt:
        # get the start position
        idx = txt.index('CLOSED_CONTOURS:')

        # number of lines
        number = int(txt[idx + 1])
        idx += 2

        # loop every closed contour
        if number > 0:
            # define data
            cn_width = []
            cn_xyz_num = []
            cn_xyz = []
            cn_label_num = []
            cn_label = []
            cn_label_xyz = []

            for _ in range(number):
                # line width
                width = float(txt[idx])
                cn_width.append(width)
                idx += 1

                # line xyz point number
                xyz_num = int(txt[idx])
                cn_xyz_num.append(xyz_num)
                idx += 1

                # line xyz
                xyz = np.array(txt[idx:(idx + 3 * xyz_num)]).astype(np.float)
                xyz.shape = [xyz_num, 3]
                cn_xyz.append(xyz)
                idx += 3 * xyz_num

                # line label
                label = txt[idx]
                cn_label.append(label)
                idx += 1

                # line label number
                label_num = int(txt[idx])
                cn_label_num.append(label_num)
                idx += 1

                # label xyz
                if label_num > 0:
                    label_xyz = np.array(
                        txt[idx:(idx + 3 * label_num)]).astype(np.float)
                    label_xyz.shape = [3, label_num]
                    cn_label_xyz.append(label_xyz)
                    idx += label_num * 3
                else:
                    cn_label_xyz.append([])

            closed_contours = {
                "cn_width": cn_width, "cn_xyz_num": cn_xyz_num,
                "cn_xyz": cn_xyz, "cn_label": cn_label,
                "cn_label_num": cn_label_num, "cn_label_xyz": cn_label_xyz}

    # ======================================================
    # read station situation
    # ======================================================
    stations = None
    if "STATION_SITUATION" in txt:
        # get the start position
        idx = txt.index('STATION_SITUATION')

        # find data subscript
        end_idx = idx + 1
        while end_idx < len(txt):
            if txt[end_idx].isdigit():
                end_idx += 1
            else:
                try:
                    f = float(txt[end_idx])
                    end_idx += 1
                except:
                    break

        if end_idx > idx + 1:
            stations = np.array(txt[(idx+1):(end_idx)])
            stations.shape = [len(stations)//2, 2]

    # ======================================================
    # read weather regions
    # ======================================================
    weather_region = None
    if "WEATHER_REGION:" in txt:
        # get the start position
        idx = txt.index('WEATHER_REGION:')

        # number of regions
        number = int(txt[idx + 1])
        idx += 2

        # loop every region
        if number > 0:
            # define data
            weather_region_code = []
            weather_region_xyz_num = []
            weather_region_xyz = []

            for _ in range(number):
                # region code
                code = int(txt[idx])
                weather_region_code.append(code)
                idx += 1

                # region xyz point number
                xyz_num = int(txt[idx])
                weather_region_xyz_num.append(xyz_num)
                idx += 1

                # region xyz point
                xyz = np.array(
                    txt[idx:(idx + 3*xyz_num)]).astype(np.float)
                xyz.shape = [xyz_num, 3]
                weather_region_xyz.append(xyz)
                idx += 3 * xyz_num

            weather_region = {
                "weather_region_code": weather_region_code,
                "weather_region_xyz_num": weather_region_xyz_num,
                "weather_region_xyz": weather_region_xyz}

    # ======================================================
    # read fill area
    # ======================================================
    fill_area = None
    if "FILLAREA:" in txt:
        # get the start position
        idx = txt.index('FILLAREA:')

        # number of regions
        number = int(txt[idx + 1])
        idx += 2

        # loop every fill area
        if number > 0:
            # define data
            fillarea_code = []
            fillarea_num = []
            fillarea_xyz = []
            fillarea_type = []
            fillarea_color = []
            fillarea_frontcolor = []
            fillarea_backcolor = []
            fillarea_gradient_angle = []
            fillarea_graphics_type = []
            fillarea_frame = []

            for _ in range(number):
                # code
                code = int(txt[idx])
                fillarea_code.append(code)
                idx += 1

                # xyz point number
                xyz_num = int(txt[idx])
                fillarea_num.append(xyz_num)
                idx += 1

                # xyz point
                xyz = np.array(
                    txt[idx:(idx + 3 * xyz_num)]).astype(np.float)
                xyz.shape = [xyz_num, 3]
                fillarea_xyz.append(xyz)
                idx += 3 * xyz_num

                # fill type
                ftype = int(txt[idx])
                fillarea_type.append(ftype)
                idx += 1

                # line color
                color = np.array(txt[idx:(idx + 4)]).astype(np.int)
                fillarea_color.append(color)
                idx += 4

                # front color
                front_color = np.array(txt[idx:(idx + 4)]).astype(np.int)
                fillarea_frontcolor.append(front_color)
                idx += 4

                # background color
                back_color = np.array(txt[idx:(idx + 4)]).astype(np.int)
                fillarea_backcolor.append(back_color)
                idx += 4

                # color gradient angle
                gradient_angle = float(txt[idx])
                fillarea_gradient_angle.append(gradient_angle)
                idx += 1

                # graphics type
                graphics_type = int(txt[idx])
                fillarea_graphics_type.append(graphics_type)
                idx += 1

                # draw frame or not
                frame = int(txt[idx])
                fillarea_frame.append(frame)
                idx += 1

            fill_area = {
                "fillarea_code": fillarea_code, "fillarea_num": fillarea_num,
                "fillarea_xyz": fillarea_xyz, "fillarea_type": fillarea_type,
                "fillarea_color": fillarea_color,
                "fillarea_frontcolor": fillarea_frontcolor,
                "fillarea_backcolor": fillarea_backcolor,
                "fillarea_gradient_angle": fillarea_gradient_angle,
                "fillarea_graphics_type": fillarea_graphics_type,
                "fillarea_frame": fillarea_frame}

    # ======================================================
    # read notes symbol
    # ======================================================
    notes_symbol = None
    if "NOTES_SYMBOL:" in txt:
        # get the start position
        idx = txt.index('NOTES_SYMBOL:')

        # number of regions
        number = int(txt[idx + 1])
        idx += 2

        # loop every notes symbol
        if number > 0:
            # define data
            nsymbol_code = []
            nsymbol_xyz = []
            nsymbol_charLen = []
            nsymbol_char = []
            nsymbol_angle = []
            nsymbol_fontLen = []
            nsymbol_fontName = []
            nsymbol_fontSize = []
            nsymbol_fontType = []
            nsymbol_color = []

            for _ in range(number):
                # code
                code = int(txt[idx])
                nsymbol_code.append(code)
                idx += 1

                # xyz
                xyz = np.array(txt[idx:(idx + 3)]).astype(np.float)
                nsymbol_xyz.append([xyz])
                idx += 3

                # character length
                char_len = int(txt[idx])
                nsymbol_charLen.append(char_len)
                idx += 1

                # characters
                char = txt[idx]
                nsymbol_char.append(char)
                idx += 1

                # character angle
                angle = txt[idx]
                nsymbol_angle.append(angle)
                idx += 1

                # font length
                font_len = txt[idx]
                nsymbol_fontLen.append(font_len)
                idx += 1

                # font name
                font_name = txt[idx]
                nsymbol_fontName.append(font_name)
                idx += 1

                # font size
                font_size = txt[idx]
                nsymbol_fontSize.append(font_size)
                idx += 1

                # font type
                font_type = txt[idx]
                nsymbol_fontType.append(font_type)
                idx += 1

                # color
                color = np.array(txt[idx:(idx + 4)]).astype(np.int)
                nsymbol_color.append(color)
                idx += 4

            notes_symbol = {
                "nsymbol_code": nsymbol_code,
                "nsymbol_xyz": nsymbol_xyz,
                "nsymbol_charLen": nsymbol_charLen,
                "nsymbol_char": nsymbol_char,
                "nsymbol_angle": nsymbol_angle,
                "nsymbol_fontLen": nsymbol_fontLen,
                "nsymbol_fontName": nsymbol_fontName,
                "nsymbol_fontSize": nsymbol_fontSize,
                "nsymbol_fontType": nsymbol_fontType,
                "nsymbol_color": nsymbol_color}

    # ======================================================
    # read lines symbols with property
    # ======================================================
    plines_symbol = None
    if "WITHPROP_LINESYMBOLS:" in txt:
        # get the start position
        idx = txt.index('WITHPROP_LINESYMBOLS:')

        # number of regions
        number = int(txt[idx + 1])
        idx += 2

        # loop every line symbol
        if number > 0:
            # define data
            plinesym_code = []
            plinesym_width = []
            plinesym_color = []
            plinesym_type = []
            plinesym_shadow = []
            plinesym_xyz_num = []
            plinesym_xyz = []
            plinesym_label = []
            plinesym_label_num = []
            plinesym_label_xyz = []

            for _ in range(number):
                # line symbol code
                code = int(txt[idx])
                plinesym_code.append(code)
                idx += 1

                # line width
                width = float(txt[idx])
                plinesym_width.append(width)
                idx += 1

                # line color
                color = np.array(txt[idx:(idx + 3)]).astype(np.int)
                plinesym_color.append([color])
                idx += 3

                # line type
                ltype = int(txt[idx])
                plinesym_type.append(ltype)
                idx += 1

                # line shadow
                shadow = int(txt[idx])
                plinesym_shadow.append(shadow)
                idx += 1

                # line symbol xyz point number
                xyz_num = int(txt[idx])
                plinesym_xyz_num.append(xyz_num)
                idx += 1

                # line symbol xyz
                xyz = np.array(txt[idx:(idx + 3 * xyz_num)]).astype(np.float)
                xyz.shape = [xyz_num, 3]
                plinesym_xyz.append(xyz)
                idx += 3 * xyz_num

                # line symbol label
                label = txt[idx]
                plinesym_label.append(label)
                idx += 1

                # line label number
                label_num = int(txt[idx])
                plinesym_label_num.append(label_num)
                idx += 1

                # label xyz
                if label_num > 0:
                    label_xyz = np.array(
                        txt[idx:(idx + 3 * label_num)]).astype(np.float)
                    label_xyz.shape = [label_num, 3]
                    plinesym_label_xyz.append(label_xyz)
                    idx += label_num * 3
                else:
                    plinesym_label_xyz.append([])

            plines_symbol = {
                "plinesym_code": plinesym_code,
                "plinesym_width": plinesym_width,
                "plinesym_color": plinesym_color,
                "plinesym_type": plinesym_type,
                "plinesym_shadow": plinesym_shadow,
                "plinesym_xyz_num": plinesym_xyz_num,
                "plinesym_xyz": plinesym_xyz,
                "plinesym_label": plinesym_label,
                "plinesym_label_num": plinesym_label_num,
                "plinesym_label_xyz": plinesym_label_xyz}

    # return data contents
    return {"file_type": 14,
            "time": time,
            "fhour": fhour,
            "lines": lines,
            "lines_symbol": lines_symbol,
            "symbols": symbols,
            "closed_contours": closed_contours,
            "stations": stations,
            "weather_region": weather_region,
            "fill_area": fill_area,
            "notes_symbol": notes_symbol,
            "plines_symbol": plines_symbol,
            "dtime":dtime,
            "data_name":data_name}



from .time_compare import time_list_mesh_error,time_list_mesh_temp,time_list_mesh_tcdc,time_list_mesh_vis,time_list_mesh_rain01h,time_list_mesh_rain03h,time_list_mesh_wind,time_list_mesh,time_list_mesh_rh,time_list_line,time_list_line_error
from .space_compare import rain_24h_sg,rain_24h_comprehensive_sg,rain_24h_comprehensive_chinaland_sg,temper_comprehensive_gg,temper_gg,temper_comprehensive_sg,temper_comprehensive_ss
from .space_compare import temper_sg,temper_ss
from .space_compare import rain_sg,rain_comprehensive_sg,rain_comprehensive_chinaland_sg,rain_comprehensive_chinaland_sl,rain_comprehensive_sl
from .score import score,score_id,score_tdt
from .table import table
from .plot import plot
from .time_space_compare import mesh_lon_time,mesh_time_lat
from .fun import get_time_str_list,get_group_name,get_x_label,get_title_from_dict,get_x_ticks,get_y_ticks,get_dayofyear_str_list,get_xun_str_list
from .error_ana_list import error_boxplot,error_boxplot_abs
from .error_ana_scatter import mae_scatter,rmse_scatter,me_scatter
from .sample_statistic import sample_tdt,sample_id
from .diurnal import diunal_max_hour_id,diunal_max_hour
from .typhoon import tran_typhoon_report



# -*-coding:utf-8-*-

from .distmap import *
from scipy.spatial import cKDTree

def cv2_distanceTransform(array_01):
    #计算每个格点到目标的最近距离
    nx = array_01.shape[1]
    ny = array_01.shape[0]
    index = np.where(array_01 ==0)
    xy = np.array(index).T
    tree = cKDTree(xy)
    xy_g = np.array(np.meshgrid(np.arange(ny),np.arange(nx))).T
    d,_ = tree.query(xy_g, k=1)
    d_g = d.reshape(ny,nx)
    return d_g



def deltametric(a, b, p=2, c=float('inf')):
    if p == float('inf') or (type(p).__name__ != "str" and str(p).isnumeric() and p > 0):
        window = boundingbox(a, b)
        a = rebound(a, window)
        b = rebound(b, window)

        #dA = cv2.distanceTransform(np.array(~(a['m'] == 1) + 0, np.uint8), cv2.DIST_L2, 3, dstType=cv2.CV_32F)
        am = np.array(~(a['m'] == 1) + 0, np.uint8)
        dA= cv2_distanceTransform(am)

        #dB = cv2.distanceTransform(np.array(~(b['m'] == 1) + 0, np.uint8), cv2.DIST_L2, 3, dstType=cv2.CV_32F)
        bm = np.array(~(b['m'] == 1) + 0, np.uint8)
        dB = cv2_distanceTransform(bm)

        if not math.isinf(c):
            dA = np.minimum(dA, c)
            dB = np.minimum(dB, c)
        if math.isinf(p):
            Z = np.abs(dA - dB)
            delta = np.max(Z)
        else:
            Z = np.abs(dA - dB) ** p
            iZ = np.mean(Z)
            delta = iZ ** (1.0 / p)
        return float(delta)
    else:
        print("p类型错误")
        raise Exception("p类型错误")



# -*- coding: utf-8 -*-
"""
Created on Wed Jan 13 11:08:18 2021

@author: 1
"""
import meteva
import copy
import numpy as np


def center_distance(grd1,grd2):
    lon1,lat1 = center_of_mass(grd1)
    lon2,lat2 = center_of_mass(grd2)
    dis  = meteva.base.math_tools.distance_on_earth_surface(lon1,lat1,lon2,lat2)
    return dis

def center_of_mass(grd):
    grid0 = meteva.base.get_grid_of_data(grd)
    data_array_2D = grd.values.squeeze()
    xd = data_array_2D.shape
    xx, yy = np.meshgrid(np.arange(xd[1]), np.arange(xd[0]))
    data_sum = np.sum(data_array_2D)
    xx_c = np.sum(data_array_2D * xx) / data_sum
    yy_c = np.sum(data_array_2D * yy) / data_sum
    lon_c = grid0.slon + xx_c * grid0.dlon
    lat_c = grid0.slat + yy_c * grid0.dlat
    return lon_c,lat_c



def intRamt(look,ob_or_fo = "ob"):
        # 提取目标的总质量，和最大值
        if ob_or_fo != "ob":
            ob_or_fo  = "fo"
        grd_data = look["grd_"+ob_or_fo].values.squeeze()  # 提取原始网格数据值
        grd_feats = look["grd_"+ob_or_fo+"_features"]      # 提取观测或目标属性
        label_count = grd_feats["label_count"]
        mass = []
        max_value = []
        lon_c = []
        lat_c = []
        mean_value = []
        grid0 = look["grid"]
        for i in range(1, label_count+ 1):
            index1 = grd_feats[i]      #提取某一个目标的对应的各个点的位置
            data = grd_data[index1]    #提取某一个目标的对应的各个点的网格值
            # 求和
            v_sum_0 = np.sum(data)
            mass.append(v_sum_0)

            #求平均值
            mean_value.append(v_sum_0 / len(data))

            # 求最大值
            v_max_0 = np.max(data)
            max_value.append(v_max_0)


            #质心经度
            lon_c_0 = np.sum(index1[1] * data) / v_sum_0
            lon_c_0 = grid0.slon + grid0.dlon * lon_c_0
            lon_c.append(lon_c_0)

            #质心纬度
            lat_c_0 = np.sum(index1[0] * data) / v_sum_0
            lat_c_0 = grid0.slat + grid0.dlat * lat_c_0
            lat_c.append(lat_c_0)


        return np.array(mass),np.array(mean_value),np.array(max_value),np.array(lon_c),np.array(lat_c)


def sal(look):

    tmp = copy.deepcopy(look)
    grid0 =tmp["grid"]

    grd_ob = tmp["grd_ob"]
    grd_fo = tmp["grd_fo"]
    #计算强度误差
    #DomR_fo = np.mean(grd_fo.values)   #计算整场降水的平均值
    #DomR_ob = np.mean(grd_ob.values)   #计算整场降水的平均值
    #A = 2 * (DomR_fo - DomR_ob)/(DomR_fo + DomR_ob)  #返回内容A

    grd_ob_label = tmp["grd_ob_label"]
    index1 = np.where((grd_ob_label.values ==0))
    grd_ob.values[index1] =0  # 将未识别成目标的零散降水值置位0

    grd_fo_label = tmp["grd_fo_label"]
    index2 = np.where((grd_fo_label.values ==0))
    grd_fo.values[index2] =0  # 将未识别成目标的零散降水值置位0

    DomR_fo = np.mean(grd_fo.values)   #计算整场降水的平均值
    DomR_ob = np.mean(grd_ob.values)   #计算整场降水的平均值
    A = 2 * (DomR_fo - DomR_ob)/(DomR_fo + DomR_ob)  #返回内容A
    dmax = meteva.base.math_tools.distance_on_earth_surface(grid0.slon,grid0.slat,grid0.elon,grid0.elat)  #计算网格对角线的长度
    cen_ob_lon,cen_ob_lat = center_of_mass(tmp['grd_ob'])   #计算观测场的整场质心
    cen_fo_lon,cen_fo_lat = center_of_mass(tmp['grd_fo'])   #计算预报场的整场质心
    dis_between_centers = meteva.base.math_tools.distance_on_earth_surface(cen_ob_lon,cen_ob_lat,cen_fo_lon,cen_fo_lat)  # 计算观测预报整场质心间距离
    L1 = dis_between_centers/dmax         #返回内容L1

    #计算各目标的质量，平均值，最大值，质心位置
    mass_ob,mean_value_ob,max_value_ob,lon_c_ob,lat_c_ob = intRamt(tmp,ob_or_fo="ob")
    mass_fo,mean_value_fo,max_value_fo,lon_c_fo,lat_c_fo  = intRamt(tmp,ob_or_fo="fo")

    #计算观测的目标质心距离观测的整场质心的距离
    center_distance_label_to_grd_ob = np.zeros(len(lon_c_ob))
    for i in range(len(lon_c_ob)):
        dis1 = meteva.base.math_tools.distance_on_earth_surface(lon_c_ob[i], lat_c_ob[i],cen_ob_lon,cen_ob_lat)
        center_distance_label_to_grd_ob[i] = dis1

    #计算预报的目标质心距离观测的整场质心的距离
    center_distance_label_to_grd_fo  = np.zeros(len(lon_c_fo))
    for i in range(len(lon_c_fo)):
        dis1 = meteva.base.math_tools.distance_on_earth_surface(lon_c_fo[i], lat_c_fo[i],cen_fo_lon,cen_fo_lat)
        center_distance_label_to_grd_fo[i] = dis1

    mass_sum_ob = np.sum(mass_ob)  # 观测目标的质量总和
    mass_sum_fo = np.sum(mass_fo) # 预报目标的质量总和
    r_ob = np.sum(mass_ob * center_distance_label_to_grd_ob)/mass_sum_ob  #各目标距离整场质心的平均距离
    r_fo = np.sum(mass_fo * center_distance_label_to_grd_fo)/mass_sum_fo  #各目标距离整场质心的平均距离

    L2 = 2 * abs(r_fo - r_ob)/dmax  #返回内容L2


    #grd_ob_label.values[grd_ob_label.values!=0] = 1
    #grd_fo_label.values[grd_fo_label.values!=0] = 1
    #L1_labeled = center_distance(grd_ob_label,grd_fo_label)/dmax   #观测和预报目标场（不区分降水长度）的质心之间的距离

    v_ob = mean_value_ob/max_value_ob  # 用 平均值/最大值 表征一致性
    v_fo = mean_value_fo/max_value_fo  # 用 平均值/最大值 表征一致性
    V_ob = np.sum(mass_ob*v_ob)/np.sum(mass_ob)    #所有目标平均的 强度一致性
    V_fo = np.sum(mass_fo*v_fo)/np.sum(mass_fo)    #所有目标平均的 强度一致性

    S = 2 * (V_fo - V_ob)/(V_fo + V_ob)

    out={
        "S": S,
        'A':A,
        "L":L1+L2,
        "L1": L1,
        "L2": L2
    }
    return out



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jan  7 23:46:15 2022

@author: tangbuxing
"""
import numpy as np
import sys


def tsboot(tseries, statistic, R, n_sim, l='NULL', sim='model',
           endcorr=True, orig_t=True, parallel=np.array(["no", "multicore", "snow"]),
           ncpus=["boot.ncpus"]):
    out = {}
    have_mc = False
    have_snow = False
    if parallel != "no" and len(ncpus) > 1:

        if (parallel == "multicore"):
            have_mc = True
        elif (parallel == "snow"):
            have_snow = True
        if (not have_mc) and (not have_snow):
            ncpus = 1

    # tscl = class(tseries)
    R = np.floor(R)
    if (R <= 0):
        sys.exit("'R' must be positive")
    call = 'match.call()'
    '''
    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))：#False
        runif(1)
    seed = get(".Random.seed", envir = .GlobalEnv, inherits = FALSE)
    t0 <- if (orig.t) 
    statistic(tseries, ...)    #暂未找到statistics函数
    else:
    none
    '''
    # ts.orig < - if (! is.matrix(tseries))
    # as.matrix(tseries)
    # if tseries is not None:
    #     ts_orig = tseries
    # n = ts_orig.shape[0]
    # if n_sim is None:
    #     n_sim = n
    #
    # # class(ts.orig) <- tscl
    #
    # if sim == "model" or sim == "scramble":
    #     l == 'NULL'
    # elif l is not 'NULL' or l <= 0 or l > n:
    #     sys.exit("invalid value of 'l'")
    #
    # fn < - if (sim == "scramble")
    # {
    #     rm(ts.orig)
    # function(r)
    # statistic(scramble(tseries, norm), ...)
    # }
    # if sim == "scramble":
    #     del (ts_orig)
    #
    #     def fn(r):

    return out
